/*
Steuerung eines Luefters zur Absenkung der Temperatur im Raum, abhaengig von der Raum- und der Aussentemperatur
  Copyright 2018  Harald Sattler

  Dieses Programm ist freie Software. Sie koennen es unter den Bedingungen der GNU General Public License, 
  wie von der Free Software Foundation veroeffentlicht, weitergeben und/oder modifizieren, entweder gemaess 
  Version 3 der Lizenz oder (nach Ihrer Option) jeder spaeteren Version.

  Die Veroeffentlichung dieses Programms erfolgt in der Hoffnung, dass es Ihnen von Nutzen sein wird, aber 
  OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT fuer 
  EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.

  Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben.
  Falls nicht, siehe <http://www.gnu.org/licenses/>.
	
  Prinzip
  Die beiden Temperaturen (Raum- und Aussentemperatur) werden mit zwei Temperatursensoren DS18B20 erfasst,
  die ueber OneWire an zwei getrennten Pins am Arduino Nano V3 angeschlossen werden.
  Ueberschreitet die Raumtemperatur einen einstellbaren Wert, wird ermittelt, ob die Aussentemperatur 
  kleiner ist. Ist das der Fall, wird der Luefter ueber ein SolidState-Relais eingeschaltet.
  Waehrend der Laufzeit des Luefters werden kontinuierlich weiter die Temperaturen erfasst und ausgewertet.
  Sinkt die Raumtemperatur unter den eingestellten Schwellwert oder steigt die Aussentemperatur zu weit an,
  wird der Luefter wieder ausgeschaltet.
  Die Schalthysterese, die Temperaturdifferenz zwischen Aussen- und Innentemperatur zum Einschalten und 
  ggf. weitere Parameter lassen sich ueber ein Menue einstellen und - bei Aenderung gegenueber dem 
  letzten Wert - persistent im EEPROM speichern.
  Ab Hardwareversion 2.3 ist ein zusaetzlicher Eingang AUX vorhanden. 
  An diesem kann ein weiterer Temperatursensor, ein DCF77-Empfangsmodul oder ein beliebiger Schalter 
  angeschlossen werden.
  
Version 0, 28.07.2017
- Rumpfprogramm erstellt, Sensor DS18B20 angesteuert, Anzeige ueber Serielle SS

Version 0, 07.08.2017
- zwei DS18B20 auslesen, Anzeige auf LOGO-LCD 10x4, Anschluss ueber I2C-Konverter mit 1602 Chip
- Einbindung LCD-Lib "NewliquidCrystal" LIB von Francisco Malpartida, https://github.com/fmalpartida/New-LiquidCrystal/releases/tag/V_1.3.4
	(muss manuell eingebunden werden, wird nicht über Librarymanager von Arduino angeboten)
- LCD.cpp fuer LOGO-Display erweitert (Cursor-Positionierung)
	Zusätzlich muss LCD.cpp (Teil der Lib) erweitert werden, damit die Cusor Positionierung für das LOGO Display passt.

	Die setCursor() Routine muss folgendermaßen geändert werden:
	-----------------------------------------------------------------------------------------
	void LCD::setCursor(uint8_t col, uint8_t row)
		{
		   const byte row_offsetsDef[]   	= { 0x00, 0x40, 0x14, 0x54 }; // For regular LCDs
		   const byte row_offsetsSpecial[]	= { 0x00, 0x40, 0x0A, 0x4A }; // For LOGO Display			<<< einfügen
		   const byte row_offsetsLarge[] 	= { 0x00, 0x40, 0x10, 0x50 }; // For 16x4 LCDs

		   if ( row >= _numlines )
		   {
			  row = _numlines-1;    // rows start at 0
		   }

		   // 16x4 LCDs have special memory map layout
		   // ----------------------------------------
		   if ( _cols == 16 && _numlines == 4 )
		   {
			  command(LCD_SETDDRAMADDR | (col + row_offsetsLarge[row]));
		   }
		   // LOGO Display (10x4) has special memory map layout											<<< einfügen
		   else if ( _cols == 10 && _numlines == 4 )	// LOGO Display									<<< einfügen
		   {																							<<< einfügen
			  command(LCD_SETDDRAMADDR | (col + row_offsetsSpecial[row]));								<<< einfügen
		   }																							<<< einfügen
		   else
		   {
			  command(LCD_SETDDRAMADDR | (col + row_offsetsDef[row]));
		   }
		}
	-----------------------------------------------------------------------------------------

Version 1, 16.08.2017
- Hauptschleife

Version 1, 10/2017
- Menu implementiert (LCDML, Nils Feldkämper) -> Anmerkung: Wird am 16.06.2018 rausgeworfen, Menü dann handgestrickt
  Info: _LCDML_CONTROL_digital_low_active ist in der Lib falsch beschrieben -> Anzeige flackert -> korrigiert
- Funktionalitaet der Regelung als Statemachine implementiert
- Anzeige der States, Animation eines Luefters, Countdown-Anzeige fuer Wartezeiten
- zwei verschiedene Luefter-Symbole definiert (einfach bei Scrollbar, schoener ohne Scrollbar)

Version 2.0, 29.10.2017
- Hardware mit zusätzlichem AUX-Anschluss fuer 3. Sensor oder Funkuhr (LOGO-Display-Adapter_2.3)
- Hardware Direktsteckung des LOGO-Display auf die Adapterplatine -> Steckerbelegung aendert sich (siehe KEY-defines)
- Wartezeit vor dem naechsten Check der Aussentemperatur zusaetzlich in grossen Schritten einstellbar (L-R-Tasten)
- Luefter im Initscreen manuell ein-/ausschaltbar (U-D-Tasten, zurueck zu Auto -> L-Taste)

Version 2.1, 03.11.2017
- Am Eingang AUX (ab Hardware 2.3) kann ein DCF77-Empfangsmodul angeschlossen werden, dessen Zeit/Datum-Information
  im InitScreen angezeigt werden kann (einstellbar ueber "Settings")
  Alle 10 Sekunden wird anstelle der Zeit kurz das Datum angezeigt
  Verwendung findet die "Arduino DCF77 Library 0.9.7" von Thijs Elenbaas, //https://github.com/thijse/Arduino-Libraries/downloads

Version 2.2, 19.11.2017
- DCF-Zeit vor dem Uebernehmen verifizieren (Parity und Validierung (Zeit(n) = Zeit(n-1) + 60s))
  Um die DCF-Zeit zu verifizieren sollte zu jeder vollen Minute das übertragene Telegramm validiert werden.
  Das kann z.B. über den Vergleich mit der zuletzt empfangenen Zeit durchgeführt werden.
  Hierbei ergeben sich aber Probleme mit diversen Übergängen und Besonderheiten, die Sprünge in den Zeiten bewirken 
  können, die dennoch ok sind (Sommer-/Winterzeit-Umstellung, Schaltjahre, Mitternacht, …).
  Für den Test auf "genau eine Minute später als das letzte Mal" reicht es, eine fortlaufende Zahl aus Datum 
  und Uhrzeit zu errechnen, die mit Mittelwerten arbeitet:
  •	Jedes Jahr hat 365 Tage (keine Schaltjahre)
  •	Jeder Monat hat 30 Tage (keine unterschiedlichen Tage je Monat)
  •	Die Rechnung beginnt z.B. mit Jahr 2000 (2017 -> Jahr = 17)
  Die Rechenvorschrift lautet also wie folgt:
  Zahl[Sekunden] = (((Jahr – 2000) * 365 * 24 * 3600) + ((Monat) * 30 * 24 * 3600) + ((Tag) * 24 * 3600) + ((Stunde) * 3600) + ((Minute) * 60) + Sekunde)
  
  Es gilt: Zeit(n) = Zeit(n-1) + 60
  Stimmt das Ergebnis nicht, wird die empfangene Zeitinformation verworfen und eine weitere Minute abgewartet.
  Der Test auf Korrektheit muss dann natürlich lauten: Zahl(n) = Zahl(n-1) + 120
  Oder allgemein: Zeit(n) = Zeit(n-1) + (60 * "bad telegrams")
  ToDo: "bad telegrams" wird bei korrektem Test auf Null gesetzt, bei fehlerhaftem Test um 1 incrementiert.
  ToDo: Die Uhrzeit muss in der Zwischenzeit im µC weitergeführt werden (in cyclic fortführen) 
  Interrupt auf beliebigem Pin -> "EnableInterrupt, a library by GreyGnome.  Copyright 2014-2015 by Michael Anthony Schwager."

Version 2.2, 24.11.2017
- Watchdog implementiert, weil hin und wieder die Anzeige einfriert und die Tasten nicht mehr reagieren.
  siehe auch: https://www.megunolink.com/articles/how-to-detect-lockups-using-the-arduino-watchdog/
  und: https://github.com/Megunolink/ArduinoCrashMonitor/tree/master/CrashTracking
25.11.2017
- "DCF77-Impuls empfangen" auf LCD signalisieren (um Stoerungen abschaetzen zu koennen)

Version 2.3, 25.11.2017
- Funktion fuer "Raum erwaermen" eingebaut.
  Im Menue kann der "Modus" auf "kuehlen" oder "waermen" gestellt werden
29.11.2017
- altes Layout per "#define LOGODisplayAdapter 2" auswaehlbar
- in den Menues wird alle 3s LCD(Z1, S8) geloescht - behoben
01.12.2017
- Sensor-Routinen fuer innen (R) und aussen (A) aufgeteilt
03.12.2017
- DCF77-Eingang frei wählbar

Version 2.4, 12.06.2018
- RTC Modul ZS-042 (DS3231) mit I2C-Anschluss programmtechnisch eingebaut.
  Die Uhrzeit wird benutzt um  Laufzeiten einstellbar zu machen (-> nachts AUS weil der Luefter zu laut ist).
  Die RTC wird mit der Adafruit "RTClib" angesteuert.
14.06.2018
- Wegen Platzmangel im Flash einige Menuepunkte entfernt, dafuer ein Menue zum Einstellen der Laufzeiten
  (Wochentage und Uhrzeiten fuer Ein-/Ausschalten. Je Wochentag kann ein eigenes Zeit-Paar gesetzt werden).
16.06.2018
- LCDML-Lib fliegt komplett raus.
  Umstellung der Logik auf eine normale Loop (das Menue nimmt zuviel Platz weg, die Uhrzeiten einstellen
  passt nicht mehr ins Flash). Der Sketch heisst jetzt "TemperaturRegler2wC.ino".
19.06.2018
- Init-Screen funktioniert wieder, DCF77 Zeit wird decodiert und angezeigt, Statusanzeigen alle in Ordnung
- Manuelle Kontrolle des Luefters im Init-Screen OK (manu ein, manu aus, auto)
- Temperatur auslesen und anzeigen, Vergleich Innen mit Aussen OK, Statemachine States anzeigen OK
- die RTClib enthaelt die Funktion unixtime(), die die unter Version 2.2 vom 19.11.2017 beschriebene,
  fortlaufende Zahl aus Datum und Uhrzeit berechnet - eigene Implementierung durch unixtime() ersetzt
23.06.2018
- Schaltzeiten einstellen funktioniert, Uhrzeit ist in der Statemachine noch nicht beruecksichtigt
- Schaltzeiten fuer Wochentage und Wochenenden werden im EEPROM persistent gespeichert
25.06.2018
- uhrzeitabhaengige Lueftersteuerung ist komplett implementiert.
  Muss gelueftet werden, aber die Uhrzeit erlaubt das nicht, wird anstelle des State in Zeile 2 kurzzeitig 
  "Zeit" eingeblendet. Das Lueftersymbol dreht in diesem Fall, aber der Luefter wird nicht eingeschaltet.
18.07.2018
- Hysterese beim Abkuehlen muss sich auf die aktuell gemessene Einschalttemperatur beziehen (ist bisher die 
  eingestellte Schwellentemperatur)
27.12.2022
- Layout Adapter Prototyp V2.0 in Eagle wieder hergestellt und Pins dafür als "LOGODisplayAdapter 1" definiert
30.12.2022
- Wenn RTC und DCF vorhanden sind und auf DCF eine gültige Uhrzeit erkannt wurde, RTC damit initialisieren

xx.07.2018
- Anschluss mehrerer Sensoren an einem Eingang

  
Version 3.x, xx.xx.2018
- Loggingfunktion fuer Ein-/Ausschaltzeitpunkte, Werte-Paare (Startzeit mit Datum + Laufzeit in Minuten)
  werden im EEPROM gespeichert und koennen ueber die serielle SS ausgelesen werden.
  Im EEPROM sind die Einstellungen ab 00 gespeichert. Ab Speicherstelle 50 koennen ca. 150 Starts abgelegt werden,
  dann kommt ein wrap around. Damit duerfte ein Sommer lang geloggt werden koennen, bevor alte Werte wieder 
  ueberschrieben werden.
  Starts werden nur protokolliert, wenn DCF77 laeuft und ein gueltiges Datum fuehrt (Jahr >= 2017)

  
Merker:
Ich muss mir mal anschauen, wieviele Bits der I2C-Converter hat. Vielleicht kann man die 6 Tasten des LOGO-Display
auch über I2C abfragen. Das wuerde den Adapter zwischen Arduino und Display deutlich vereinfachen.
Der Adapter hat aber nur 8 Bit und die Ausgabe auf das Display benoetigt schon mindestens 6(?) davon, eher mehr.
Passend mit 16 Bit IO: PI4IOE5V9539 
(https://www.diodes.com/products/connectivity-and-timing/interface/interface-logic/io-expanders/part/PI4IOE5V9539)
*/

// Defines für Debugging
//#define DEBUGON				// einige Debugausgaben auf Serial
//#define DEBUGTASTEN			// Druck auf die Tasten loggen auf Serial
//#define NOSENSORS				// wenn keine Temperatursensoren angeschlossen sind, alle Aufrufe deaktivieren

  // include libs
  #include <I2CIO.h>
  #include <OneWire.h>
  #include <LiquidCrystal_I2C.h>	// Malpartida Lib "New-LiquidCrystal" mit Änderungen bei setCursor() (siehe oben)
  #include <EEPROM.h>
  #include <avr/eeprom.h>
  #include <avr/wdt.h>
  #include <RTClib.h>
  
// Version von TempReg (TemperaturRegler2wC.ino)
  #define TempRegVersionMajor 2  
  #define TempRegVersionMinor 4  
  #define LOGODisplayAdapter 3	// Hardware der Display-Platine
								// Prototyp 2.0: 1, erster Entwurf 2.1 + 2.2: 2, nachfolgende: 3
  
// ********************************************************************* 
// display TYPE 
// *********************************************************************
  // settings for lcd 
  #define DISPLAY_cols             10
  #define DISPLAY_rows             4  

// Adresse des I2C-parallel Converters  
#define I2C_Address 0x3F

  // lcd object
LiquidCrystal_I2C lcd(I2C_Address, 2, 1, 0, 4, 5, 6, 7);  // set LCD I2C address and pin functions

// Lueftersteuerung Ventilator definieren (Sonderzeichen im Display-RAM) - "schoene" Version des Luefters
  const uint8_t ventilator[4][8] = {
    {B00000, B00111, B00110, B00100, B01100, B11100, B00000, B00000}, // Ventilator state 1 + 5
    {B00000, B00010, B00011, B11111, B11000, B01000, B00000, B00000}, // Ventilator state 2 + 6
    {B00000, B10000, B11000, B11111, B00011, B00001, B00000, B00000}, // Ventilator state 3 + 7
    {B00000, B01100, B11100, B00100, B00111, B00110, B00000, B00000}, // Ventilator state 4 + 8
  }; 
// DCF77 Antenne definieren (Sonderzeichen im Display-RAM)
  const uint8_t antenne[3][8] = {
	// Alternative 1: Suchen -> wechselndes Kreuz am oberen Ende der Antenne
    // {B00100, B01010, B10101, B01010, B00100, B00100, B01110, B00000}, // Antenne suchend - 1
    // {B10001, B01010, B00100, B01010, B10101, B00100, B01110, B00000}, // Antenne suchend - 2
	// Alternative 2: Suchen -> grosser/kleiner Kreis am oberen Ende der Antenne
    {B01110, B10001, B10101, B01110, B00100, B00100, B01110, B00000}, // Antenne suchend - 1 und gefunden
    {B00000, B01110, B01110, B00100, B00100, B00100, B01110, B00000}, // Antenne suchend - 2
    {B00000, B00000, B00100, B00100, B00100, B00100, B01110, B00000}, // Antenne suchend - 3
 // {B01110, B10001, B10101, B01110, B00100, B00100, B01110, B00000}, // Antenne gefunden, unnoetig, bei 1. enthalten
  }; 

// Defines
	#define OFF 		0	// 
	#define ON			1
	#define LED13 		13

// defines fuer Tasten des LOGO-Display am Arduino (!!! Version mit Direktsteckung !!!)
// fuer die Direktsteckung musste der Stecker umgedreht werden, was ein komplettes Umrouten
// der Leitungen und Belegung zwischen Stecker und µC bzw. I2C-Adapter bedingte (V2.2 -> V2.3)
#if (LOGODisplayAdapter == 3) 	// Hardware der Display-Platine - erster Entwurf: 2, nachfolgende: 3
	// -------------------------------------------------------------------------------------------	
	// Belegung fuer V2.3 und hoeher
	// -------------------------------------------------------------------------------------------	
	// Inputs
	#define KEY_RIGHT 	3
	#define KEY_DOWN 	4
	#define KEY_OK 		5
	#define KEY_ESC 	6
	#define KEY_LEFT 	7
	#define KEY_UP 		8 
	// Outputs
	#define RELAIS 		9	// fuer Version V2.3 (!!! Version mit Direktsteckung !!!)
	// defines fuer Temperatur Sensoren, Pins am Arduino
	#define RaumSensor 10
	#define AussenSensor 11
	/** Where is the DCF receiver connected? */
	#define DCF77PIN 12
	// Version der AdapterPlatine
	#define LOGOAdapterPlatine "V2.3+"		// Anzeige der AdapterVersion unter "Settings"
	// -------------------------------------------------------------------------------------------	
#elif  (LOGODisplayAdapter == 2) 
// defines fuer Tasten des LOGO-Display am Arduino (!!! Version mit Flachbandleitung !!!)
	// -------------------------------------------------------------------------------------------	
	// Belegung fuer V2.1 bis V2.2 (und V2 == das erste Modul ohne AUX)
	// -------------------------------------------------------------------------------------------	
	// Inputs
	#define KEY_RIGHT 	6
	#define KEY_DOWN 	7
	#define KEY_OK 		5
	#define KEY_ESC 	4
	#define KEY_LEFT 	3
	#define KEY_UP 		2 
	// Outputs
	#define RELAIS 		8
	// defines fuer Temperatur Sensoren, Pins am Arduino
	#define RaumSensor 10
	#define AussenSensor 9
	/** Where is the DCF receiver connected? */
	#define DCF77PIN 11
	// Version der AdapterPlatine
	#define LOGOAdapterPlatine "V2.1..2.2"	// Anzeige der AdapterVersion unter "Settings"
	// -------------------------------------------------------------------------------------------	
#elif  (LOGODisplayAdapter == 1) 
// defines fuer Tasten des LOGO-Display am Arduino (!!! Version mit Flachbandleitung !!!)
	// -------------------------------------------------------------------------------------------	
	// Belegung fuer V2.1 bis V2.2 (und V2 == das erste Modul ohne AUX)
	// -------------------------------------------------------------------------------------------	
	// Inputs
	#define KEY_RIGHT 	6
	#define KEY_DOWN 	7
	#define KEY_OK 		5
	#define KEY_ESC 	4
	#define KEY_LEFT 	3
	#define KEY_UP 		2 
	// Outputs
	#define RELAIS 		8
	// defines fuer Temperatur Sensoren, Pins am Arduino
	#define RaumSensor 10
	#define AussenSensor 9		// Pin 9 ist nicht angeschlossen, Dummy weil DCF77 Modul auf Pin 11 liegt
	/** Where is the DCF receiver connected? */
	#define DCF77PIN 11					// es gibt kein AUX, daher Doppel-/Alternativbelegung mit Aussen
	// Version der AdapterPlatine
	#define LOGOAdapterPlatine "V2.0"	// Anzeige der AdapterVersion unter "Settings"
	// -------------------------------------------------------------------------------------------	
#endif	

// nachfolgende #defines gelten, wenn die Bedienung ueber Tasten (Joystick oder auch einzelne 
// Touch-Sensoren (siehe Link oben)) erfolgt
#pragma message("Normale Tasten werden verwendet...")	// Informative Meldung beim Compilieren
#define QTUP KEY_UP				// DataPin der Tasten am Arduino 
#define QTDOWN KEY_DOWN
#define QTLEFT KEY_LEFT
#define QTRIGHT KEY_RIGHT
#define QTENTER KEY_OK
#define QTESC KEY_ESC
#define GETKEYMACRO(RICHTUNG) (digitalRead(RICHTUNG) == LOW)	// je nach Typ der Bedienung

char cChangePos = 0;					// welcher Parameter ist im Menue selektiert
char clastChangePos = 0;

// Merker fuer Tasten
bool bKeyRightTrig = false;
bool bKeyLeftTrig = false;
bool bKeyUpTrig = false;
bool bKeyDownTrig = false;
bool bKeyEnterTrig = false;
bool bKeyEscTrig = false;
bool bLongKeyRight = false;
unsigned long lRightKeyPressed = 0;		// Timervar für tastendruck
unsigned long lRightKeyLongPressed = 0;	// Timervar für tastendruck

// Variablen fuer Menuesteuerung
bool inMenu = false;					// Loop durchlaeuft Menue wenn true
bool bFirstTimeInMenu = true;			// Eintritt ins Menue mit KEYENTER
unsigned long lMenuTimeout;				// Menue nach Ablauf automatisch wieder verlassen
#define AUTOLEAVEMENU 15000				// Wartezeit fuer Auto-Menu stop (15 Sekunden)

// DS18B20 Temperaturchip i/o
OneWire TempR(RaumSensor);  	// Raumtemperatur an pin 10
OneWire TempA(AussenSensor);  	// Aussentemperatur an pin 11

// Variablen fuer Temperatursensoren auslesen
int HighByte, LowByte, TReading, SignBit, Tc_100, Whole, Fract;
int16_t raw;
byte cfg;
byte i;
byte present = 0;
byte data[12];
byte addrR[8];
byte addrA[8];

bool bCoolWarm;						// wird ueber Fragezeichen-Operator fuer kuehlen/waermen gesetzt
float tempRaum, tempAussen, tempRaumAbs, tempAussenAbs;
float tempRaumEIN;					// Einschalttemperatur merken für Hysterese
char tempbuff[8] = "";				// Hilfspuffer
char tempAussenTxt[8] = "";
char tempRaumTxt[8] = "";

// globale Variablen
bool bConversionFlag = false;		// Wandlung in den Sensoren laeuft
unsigned long uConversionTimer = 0;	// Wandlung dauert 750 ms, so lange also Sensoren nicht ansprechen

char cGlobalState;					// globaler Merker fuer aktuellen State der Steuerung
unsigned long uStateAnzeige = 0;	// Flags und Zeiten fuer State-Anzeige
bool bStateDispOn = false;			// fuer Aufblitzen des StateValue in der Anzeige
bool bStateOnInInit = false;		// State nur im Initscreen anzeigen
bool bState1FromManu = false;		// "auto" wird sonst sofort wieder geloescht, das soll verhindert werden...
unsigned long uState1FromManuTimer = 0;	// ... mit timeout

char cTmpLuefter;					// globale Variable fuer Darstellung Luefter (/-\|)
unsigned long uLuefterDispTimer;	// Darstellung des Luefters drehen lassen
bool bShowLuefter = false;			// Lueftersymbol in Anzeige sichtbar
int iLuefterDispState = 1;			// einer von vier States (siehe cTmpLuefter)
#define LUEFTERDISPPHASE 75			// wie schnell dreht das Lueftersymbol (Intervall [ms])

char cTmpAntenne;					// globale Variable fuer Darstellung Antenne
unsigned long uAntenneDispTimer;	// Darstellung der Antenne animieren
bool bShowAntenne = false;			// Antennensymbol in Anzeige sichtbar
unsigned char ucAntenneDispState = 1;// einer von zwei States (cTmpAntenne) beim Suchen oder statisch wenn gefunden
unsigned char ucDCF77changed = 0;	// wurde bereits eine gueltige UZ-Info empfangen?
#define ANTENNEDISPPHASE 300		// wie schnell blinkt das Antennensymbol (Intervall [ms])
bool bDcfParity = false;			// true wenn Zeit korrekt empfangen wurde
bool bDcfTimeOk = false;			// true wenn die Zeit korrekt inkrementiert wurde (Zeit(n) = Zeit(n-1) + 60s)

unsigned long uCheckVorlaufTimer;	// Timer fuer CheckVorlauf
unsigned long uNextCheckTimer;		// Timer fuer WaitBeforeNextCheck
unsigned long uCountdownTimer;		// Timer fuer Anzeige Countdown
unsigned long llastTempGetTime;		// Timer fuer Auslesen der Temperatursensoren
unsigned long lloopTimer;			// loop gleichmaessig durchlaufen
unsigned long uZeitBlinker;			// Anzeige-Timer fuer "Zeit" wenn Luefter nachts nicht laufen darf

unsigned long lAktivityBlinker = 0;	// LED13 blinkt im Sekundentakt um Haenger zu erkennen

bool valueChanged = false;			// globaler Merker, ob ins EEPROM gespeichert werden soll
int iFirstTimeStarted = 0xFFFF;		// Arduino jungfraeulich? Ja: FFFF - Nein: AA55

// per Menu aenderbare Parameter
float fSollTemp = 27.0;				// [°], initiale maximale Solltemperatur im Raum
int iTempDiff = 2;					// [K], wieviel Kelvin muss aussen kuehler sein, damit Luefter laeuft
int iTempCheckVorlauf = 60;			// [s], wie lange soll der Lüfter die Aussenluft ansaugen, 
									// 		bevor die Aussenlufttemperatur gemessen wird
int iWaitBeforeNextCheck = 1800;	// [s], 600s = 10min, wenn es aussen zu warm ist, wie lange soll vor 
									// 		der naechsten Pruefung gewartet werden
int iTempCheckInterval = 1;			// [s], zur Laufzeit wird die Temperatur innen alle xx Sekunden gemessen
int iTempHyst = 2;					// [K], Hysterese bei der Temperaturmessung innen
bool bCooling = true;				// Soll innen gekuehlt oder erwaermt werden? Default: kuehlen
bool bDCF77 = false;				// Soll die DCF-Zeit im InitScreen angezeigt werden? Default: nein
bool bRTCtime = false;				// Soll die RTC-Zeit im InitScreen angezeigt werden? Default: nein
bool binTimerScreen = false;		// damit der Cursor im Timer-Einstellscreen gezeigt wird, darf der 
									// Screen nur einmal aufgebaut werden
unsigned char cCurX = 1, cCurY = 1;	// Cursor-Koordinaten im Timer-Einstellscreen
unsigned char cSensorPos = 1;		// welcher Sensor? [Innen, Aussen, Wechselrichter]


#define CHANGEPOSMAX 18				// letzter Menuepunkt
// temporaere Parameter im Menue
float fTempMaxRoomtempval;			// globaler Merker des Wertes (fuer Cancel)
int iTempDifftempval;				// globaler Merker des Wertes (fuer Cancel)
int iTmpHysterese;					// globaler Merker des Wertes (fuer Cancel)
int iTmpVorlauf;					// globaler Merker des Wertes (fuer Cancel)
int iTmpPause;						// globaler Merker des Wertes (fuer Cancel)
int iTmpInterval;					// globaler Merker des Wertes (fuer Cancel)
unsigned char cTmpSensorPos;		// globaler Merker des Wertes (fuer Cancel)
// Merker fuer Laufzeiten... TBD
bool bTmpCooling;					// globaler Merker des Wertes (fuer Cancel)
bool bTmpDCF77;						// globaler Merker des Wertes (fuer Cancel)
bool bTmpRTCtime;					// globaler Merker des Wertes (fuer Cancel)
bool bTmpLuefterMitUhrSteuern;		// globaler Merker des Wertes (fuer Cancel)

// Timer-Screen Variablen
bool bmdmdfBlockSel = false;
bool bssBlockSel = false;

// Lueftersteuerung abhaengig von der Uhrzeit
struct LuefterZeit {
  unsigned int StartStunde;			// Start-Uhrzeit
  unsigned int StartMinute;
  unsigned int EndeStunde;			// Ende-Uhrzeit
  unsigned int EndeMinute;
  unsigned int StartMinAbMidnight;	// Start-Uhrzeit in Minuten ab Mitternacht
  unsigned int EndeMinAbMidnight;	// Ende-Uhrzeit in Minuten ab Mitternacht
};
struct LuefterZeit WoTage, WoEnde;	// 2 Strukturen mit Start- und Endezeiten für Wochentage und Wochenende
bool bLuefterErlaubt = true;		// der Luefter darf laufen, Flag wird anhand der Uhrzeit gesetzt
bool bLuefterMitUhrSteuern = false;	// Uhrzeitsteuerung Ein/Aus (im EEPROM)
unsigned int iaktZeitMinAbMidnight;	// aktuelle Uhrzeit in Minuten nach Mitternacht

int iTmpChkVorlauf;					// fuer Countdown-Anzeige im Display
int iTmpWaitNextChk;

//{ DCF77 Stuff - start +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define EI_ARDUINO_INTERRUPTED_PIN
#include <EnableInterrupt.h>

/** Where is the LED connected? */
#define BLINKPIN 13
/** Turn debugging on or off  */
#undef DCF_DEBUG
// #define DCF_DEBUG 1

/** Number of milliseconds to elapse before we assume a "1",
 * if we receive a falling edge before - its a 0 */
#define DCF_split_millis 140
/** There is no signal in second 59 - detect the beginning of a new minute. */
#define DCF_sync_millis 1200

/**
 * Definitions for the timer interrupt 2 handler:
 * The Arduino runs at 16 Mhz, we use a prescaler of 64 -> We need to 
 * initialize the counter with 6. This way, we have 1000 interrupts per second.
 * We use tick_counter to count the interrupts.
 */
#define INIT_TIMER_COUNT 6
#define RESET_TIMER2 TCNT2 = INIT_TIMER_COUNT
int tick_counter = 0;
int TIMSK;
int TCCR2;
int OCIE2;

//{ defines fuer "EnableInterrupt"-Library (start) 
/* // die Ausgaben auf Serial sind unnötig, lassen sich aber innerhalb des #define nicht auskommentieren
#define setupInterrupt(x) \
  EI_printPSTR("Add pin: "); \
  EI_printPSTR(#x); \
  EI_printPSTR("\r\n"); \
  pinMode( x, INPUT_PULLUP); \
  enableInterrupt( x, interruptFunction, CHANGE)
*/
#define setupInterrupt(x) \
  pinMode( x, INPUT_PULLUP); \
  enableInterrupt( x, interruptFunction, CHANGE)
//} defines fuer "EnableInterrupt"-Library (end)

/** DCF time format struct */
struct DCF77Buffer {
  unsigned long long prefix	:21;
  unsigned long long Min	:7;	// minutes
  unsigned long long P1		:1;	// parity minutes
  unsigned long long Hour	:6;	// hours
  unsigned long long P2		:1;	// parity hours
  unsigned long long Day	:6;	// day
  unsigned long long Weekday	:3;	// day of week
  unsigned long long Month	:5;	// month
  unsigned long long Year	:8;	// year (5 -> 2005)
  unsigned long long P3		:1;	// parity
};
struct {
	unsigned char parity_flag	:1;
	unsigned char parity_min	:1;
	unsigned char parity_hour	:1;
	unsigned char parity_date	:1;
} flags;

/** Clock variables  */
volatile unsigned char DCFSignalState = 0;  
unsigned char previousSignalState;
unsigned long previousFlankTime;
int bufferPosition;
unsigned long long dcf_rx_buffer;

/** time vars: the time is stored here! */
// Trick
RTC_Millis laufendeUhrzeit;
DateTime aktUhr;
// original
volatile unsigned char ss;
volatile unsigned char mm;
volatile unsigned char hh;
volatile unsigned char day;
volatile unsigned char mon;
volatile unsigned int year;
volatile unsigned char ucAktSekunde;		// hiermit wird die laufende Sekunde im Display angezeigt
volatile unsigned long ulDatumZeit;			// aktuelles Datum und Zeit als fortlaufende Zahl (vereinfacht, siehe Einleitung)
volatile unsigned long ulLastDcfDatumZeit;	// letztes gueltiges Datum und Zeit
volatile unsigned long ulAntenneTimer;		// nach 3 s ohne DCF77-Signal Antenne ohne Ring anzeigen, "kein DCF-Empfang"
volatile unsigned char ucAntenneState = 0;	// default: keine DCF-Signale empfangen
    
volatile uint8_t externalInterruptFlag = 0;
volatile uint8_t pinChangeInterruptFlag = 0;
volatile bool dcfImpuls = false;			// IRQ wurde durchlaufen
bool dcfImpulsIsOn = false;					// Impuls-Anzeige im Display ist EIN
unsigned long dcfImpulsDauer = 0;			// zum Check auf Impulsabstand fuer die Anzeige
bool dcfDebug = false;						// Anzeige von Debug-Hilfen

// -------------------------------------------------------------------------------
// Interrupt function fuer Pin DCF77PIN
// -------------------------------------------------------------------------------
void interruptFunction () 
{
  // pinChangeInterruptFlag = arduinoInterruptedPin;
  DCFSignalState = !arduinoPinState;		// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<	Signal ist negiert (aktive low)
											// Polaritaet einstellen in scanSignal() (~Z.546)
  dcfImpuls = true;	
}

/** used in main loop: detect a new second... */
unsigned char previousSecond;
    
// -------------------------------------------------------------------------------
/** Initialize the DCF77 routines: initialize the variables,
 * configure the interrupt behaviour */
// -------------------------------------------------------------------------------
void DCF77Init() {
  previousSignalState=0;
  previousFlankTime=0;
  bufferPosition=0;
  dcf_rx_buffer=0;
  ss=mm=hh=day=mon=year=0;
#ifdef DCF_DEBUG 
  Serial.println(F("Initializing DCF77 routines"));
  Serial.print(F("Using DCF77 pin #"));
  Serial.println(DCF77PIN);
#endif
  pinMode(BLINKPIN, OUTPUT);
  pinMode(DCF77PIN, INPUT);		// D12 oder D11
  
#ifdef DCF_DEBUG
  Serial.println(F("Initializing timerinterrupt"));
#endif
  //Timer2 Settings: Timer Prescaler /64, 
  TCCR2 |= (1<<CS22);    // turn on CS22 bit
  TCCR2 &= ~((1<<CS21) | (1<<CS20));    // turn off CS21 and CS20 bits   
  // Use normal mode
  TCCR2 &= ~((1<<WGM21) | (1<<WGM20));   // turn off WGM21 and WGM20 bits 
  // Use internal clock - external clock not used in Arduino
  ASSR |= (0<<AS2);
  TIMSK |= (1<<TOIE2) | (0<<OCIE2);        //Timer2 Overflow Interrupt Enable  
  RESET_TIMER2;
}

// -------------------------------------------------------------------------------
/** Append a signal to the dcf_rx_buffer. Argument can be 1 or 0. An internal
   counter shifts the writing position within the buffer. 
   If position > 59, a new minute begins -> time to call finalizeBuffer() */
// -------------------------------------------------------------------------------
void appendSignal(unsigned char signal) {
#ifdef DCF_DEBUG
  Serial.print(F(", appending value "));
  Serial.print(signal, DEC);
  Serial.print(F(" at position "));
  Serial.println(bufferPosition);
#endif
  dcf_rx_buffer = dcf_rx_buffer | ((unsigned long long) signal << bufferPosition);
  // Update the parity bits. First: Reset when minute, hour or date starts.
  if (bufferPosition ==  21 || bufferPosition ==  29 || bufferPosition ==  36) {
	flags.parity_flag = 0;
  }
  // save the parity when the corresponding segment ends
  if (bufferPosition ==  28) {flags.parity_min = flags.parity_flag;};
  if (bufferPosition ==  35) {flags.parity_hour = flags.parity_flag;};
  if (bufferPosition ==  58) {flags.parity_date = flags.parity_flag;};
  // When we received a 1, toggle the parity flag
  if (signal == 1) {
    flags.parity_flag = flags.parity_flag ^ 1;
  }
  bufferPosition++;
  if (bufferPosition > 59) {
    finalizeBuffer();
  }
  // Sekunde ausgeben auf LCD
  ucAktSekunde = bufferPosition;
}

// -------------------------------------------------------------------------------
/** Evaluates the information stored in the buffer. This is where the DCF77
 * signal is decoded and the internal clock is updated */
// -------------------------------------------------------------------------------
void finalizeBuffer(void) {
  if (bufferPosition == 59) 
  {
#ifdef DCF_DEBUG
    Serial.println(F("Finalizing Buffer"));
#endif
    struct DCF77Buffer *rx_buffer;
    rx_buffer = (struct DCF77Buffer *)(unsigned long long)&dcf_rx_buffer;
    if (flags.parity_min == rx_buffer->P1  &&
        flags.parity_hour == rx_buffer->P2  &&
        flags.parity_date == rx_buffer->P3) 
    {
	  bDcfParity = true;		// Parity OK -> Zeit uebernehmen
#ifdef DCF_DEBUG
	  Serial.println(F("Parity check OK - updating time."));
#endif
	  ulLastDcfDatumZeit = ulDatumZeit; 					// fuer Vergleich merken
	  
	  // convert the received bits from BCD
	  mm   = rx_buffer->Min-((rx_buffer->Min/16)*6);
	  hh   = rx_buffer->Hour-((rx_buffer->Hour/16)*6);
	  day  = rx_buffer->Day-((rx_buffer->Day/16)*6); 
	  mon  = rx_buffer->Month-((rx_buffer->Month/16)*6);
	  year = rx_buffer->Year-((rx_buffer->Year/16)*6) + 2000;
	  
	  laufendeUhrzeit.adjust(DateTime((uint16_t) (2000 + rx_buffer->Year-((rx_buffer->Year/16)*6)),
							(uint8_t) rx_buffer->Month-((rx_buffer->Month/16)*6),
							(uint8_t) rx_buffer->Day-((rx_buffer->Day/16)*6),
							(uint8_t) rx_buffer->Hour-((rx_buffer->Hour/16)*6), 
							(uint8_t) rx_buffer->Min-((rx_buffer->Min/16)*6),
							(uint8_t) ucAktSekunde));
	  

	  // Datum und Zeit in eine fortlaufende Zahl wandeln [Sekunden]
	  // ulDatumZeit = (((year - 2000) * 365 * 24 * 3600) + (mon * 30 * 24 * 3600) + (day * 24 * 3600) + (hh * 3600) + (mm * 60));
	  // ulDatumZeit = (((((year - 2000) * 8760) + (mon * 720) + (day * 24) + hh) * 3600) + (mm * 60));
	  // ulDatumZeit = ((((year - 2000) * 31536000) + (mon * 2592000) + (day * 86400) + hh) + (mm * 60));
	  ulDatumZeit = aktUhr.unixtime();

	  
	  if(abs(ulDatumZeit - (ulLastDcfDatumZeit + 60)) < 1)
	  {
		  bDcfTimeOk = true;	// OK: alte und neue Zeit unterscheiden sich um genau 60 Sekunden -> uebernehmen!
	  }
	  else
	  {
		  bDcfTimeOk = false;	// NOK: alte und neue Zeit unterscheiden sich nicht um genau 60 Sekunden -> nicht uebernehmen!
	  }
    }
	else 
	{
		bDcfParity = false;		// Parity error aufgetreten -> Zeit nicht uebernehmen
		bDcfTimeOk = false;		// Parity NOK -> Time NOK
#ifdef DCF_DEBUG
        Serial.println(F("Parity check NOK - running on internal clock."));
#endif

    }
  } 
  
  aktUhr = laufendeUhrzeit.now();
  // Serial.println(ulDatumZeit);
  
  // reset stuff
  ss = 0;
  bufferPosition = 0;
  dcf_rx_buffer = 0;
}

// -------------------------------------------------------------------------------
/** Dump the time to the serial line */
// -------------------------------------------------------------------------------
#ifdef DCF_DEBUG
void serialDumpTime(void){
  Serial.print(F("Time: "));
  Serial.print(hh, DEC);
  Serial.print(F(":"));
  Serial.print(mm, DEC);
  Serial.print(F(":"));
  Serial.print(ss, DEC);
  Serial.print(F(" Date: "));
  Serial.print(day, DEC);
  Serial.print(F("."));
  Serial.print(mon, DEC);
  Serial.print(F("."));
  Serial.println(year, DEC);
}
#endif


// -------------------------------------------------------------------------------
/** Evaluates the signal as it is received. Decides whether we received
 * a "1" or a "0" based on the duration of the signal */
// -------------------------------------------------------------------------------
void scanSignal(void){ 
    // if (DCFSignalState == 1) {			// hier Polarität des DCF-Signals einstellen
    if (DCFSignalState == 0) {				// == 0 fuer active low Signal
      /* the leading edge... */
      unsigned long thisFlankTime = millis();
      if ((thisFlankTime - previousFlankTime) > DCF_sync_millis) {
#ifdef DCF_DEBUG
        serialDumpTime();
        Serial.println(F("####"));
        Serial.println(F("#### Begin of new Minute!!!"));
        Serial.println(F("####"));
#endif
        finalizeBuffer();
      }
      previousFlankTime = thisFlankTime;
#ifdef DCF_DEBUG
      Serial.print(previousFlankTime);
      Serial.print(F(": DCF77 Signal detected, "));
#endif
    } 
    else {
      /* ... the trailing  flank */
      unsigned long difference = (millis() - previousFlankTime);
#ifdef DCF_DEBUG
      Serial.print(F("duration: "));
      Serial.print(difference);
#endif
	  if (difference < DCF_split_millis) {
		appendSignal(0);
	  } 
	  else {
		appendSignal(1);
	  }
	}
}

// -------------------------------------------------------------------------------
// * The interrupt routine for counting seconds - increment hh:mm:ss */
// -------------------------------------------------------------------------------
ISR(TIMER2_OVF_vect) {
  RESET_TIMER2;
  tick_counter += 1;
  if (tick_counter == 1000) {
    ss++;
    if (ss==60) {
      ss=0;
      mm++;
      if (mm==60) {
        mm=0;
        hh++;
        if (hh==24) 
          hh=0;
      }
    }
    tick_counter = 0;
  }
};
//} DCF77 Stuff - end +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// { RTClock ZA-042 (DS3231) Stuff
RTC_DS3231 rtcDS3231;
// char daysOfTheWeek[7][12] = {"Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"};


  
// *********************************************************************
// SETUP
// *********************************************************************
  void setup()
  { 
	// Watchdog sofort ausschalten, damit setup() nicht unterbrochen wird
	wdt_disable();
	
    // serial init; only needed if serial control is used 							
    while(!Serial);                    // wait until serial ready                   
    Serial.begin(57600);               // start serial                              
                                                                                    
	// define IO mode of pins                                                       
	pinMode(RELAIS, OUTPUT);                                                        
	pinMode(LED13, OUTPUT);
	
	pinMode(RaumSensor, INPUT_PULLUP);
	pinMode(AussenSensor, INPUT_PULLUP);
	
	pinMode(KEY_RIGHT 	, INPUT_PULLUP);
	pinMode(KEY_DOWN 	, INPUT_PULLUP);
	pinMode(KEY_OK 		, INPUT_PULLUP);
	pinMode(KEY_ESC 	, INPUT_PULLUP);
	pinMode(KEY_LEFT 	, INPUT_PULLUP);
	pinMode(KEY_UP 		, INPUT_PULLUP);


	// AUX-Eingang siehe Line 376:   pinMode(DCF77PIN, INPUT);		// D12

	// LCD Begin
    lcd.begin(DISPLAY_cols,DISPLAY_rows);  
    
    // set special chars for ventilator
    lcd.createChar(1, (uint8_t*)ventilator[0]);			// komplexer Ventilator mit Flaechen
    lcd.createChar(2, (uint8_t*)ventilator[1]);
    lcd.createChar(3, (uint8_t*)ventilator[2]);
    lcd.createChar(4, (uint8_t*)ventilator[3]);
    // set special chars for antenne
    lcd.createChar(5, (uint8_t*)antenne[0]);			// DCF77-Antennensymbol
    lcd.createChar(6, (uint8_t*)antenne[1]);
    lcd.createChar(7, (uint8_t*)antenne[2]);

	// OneWire fuer Temperatursensoren initialisieren
#ifndef NOSENSORS
	initOneWireR();
	initOneWireA();
#endif
	// DCF77 initialisieren
	setupInterrupt(DCF77PIN);		// attach interrupt for DCF77 on pin D12 
	DCF77Init();					// DCF decoder housekeeping
	
	// Arduino jungfraeulich?
	int eeAddress = 0;
	EEPROM.get(eeAddress, iFirstTimeStarted);
	if(iFirstTimeStarted == 0xFFFF) 			// FFFF = jungfraeulich
	{
		// LCDML_DISP_jumpToFunc(LCDML_FUNC_initial_EEPROM); // EEPROM initialisieren direkt aufrufen
	} 
	else 
	{				// nein, lief schon mal, EEPROM-Werte sind gesetzt
		readFromEEPROM();		// RAM-Variablen aus dem EEPROM vorbelegen
		
		// Uhrzeiten checken und ggf. auf Defaults setzen
		if(WoTage.StartStunde > 23) WoTage.StartStunde =  6;
		if(WoTage.StartMinute > 59) WoTage.StartMinute =  0;
		if(WoTage.EndeStunde  > 23) WoTage.EndeStunde  = 22;
		if(WoTage.EndeMinute  > 59) WoTage.EndeMinute  =  0;
		
		if(WoEnde.StartStunde > 23) WoEnde.StartStunde =  8;
		if(WoEnde.StartMinute > 59) WoEnde.StartMinute =  0;
		if(WoEnde.EndeStunde  > 23) WoEnde.EndeStunde  = 22;
		if(WoEnde.EndeMinute  > 59) WoEnde.EndeMinute  =  0;
	}
	
	
	wdt_enable(WDTO_4S);		// Aktiviere Watchdog mit 4s Zeitkonstante
	bStateDispOn = true;		// InitScreen anzeigen
  }

// *********************************************************************
// LOOP
// *********************************************************************
void loop()
{ 
	// Watchdog retriggern (falls nicht: Nach 4 Sekunden wird der Controller neu gestartet)
	wdt_reset();

	// !!! nachfolgende Zeile muss im Betrieb unbedingt auskommentiert sein !!!
	// digitalWrite(RELAIS, !digitalRead(RELAIS));	// toggle Ausgang zur Schleifenzeitbestimmung
	
	// Test - um die Looptime annaehernd konstant zu halten hier eine Bremse...
	// while(millis() < lloopTimer + 20) {} 
	// lloopTimer = millis();
	// klappt (so) nicht :(

// ------------------------------------------------------------------------------------------------------------------
	// Tastendruck RECHTS fuehrt ins Menue
	// ----------------------------------------------------------------------------------------------------------------------------
	if(GETKEYMACRO(QTENTER)) inMenu = true;	

	if (!inMenu) 					// normaler Betrieb
	{
// ------------------------------------------------------------------------------------------------------------------
		//{ DCF77-Impuls visualisieren START
// ------------------------------------------------------------------------------------------------------------------
		if(dcfImpuls == true) 					// im IRQ wird das Flag gesetzt
		{
#ifdef DEBUGON
			Serial.print(F("DCF Impuls\n"));
#endif
			ulAntenneTimer = millis();			// aktuelle Zeit merken. Wenn 3s rum, "kein DCF-Empfang" anzeigen
			
			if(ucAntenneState == 0) ucAntenneState = 1;	// wenn DCF-Signale empfangen -> Antenne ohne Ringe anzeigen

			if(bStateOnInInit == true)				// Impuls/Blank nur ausgeben, wenn im Init-Screen
			{
				if(dcfImpulsIsOn == false)
				{ 	
					dcfImpulsDauer = millis();
					
					if(ucAntenneState < 3) 			// Impuls nur anzeigen, wenn Zeit noch nicht validiert ist
					{
						lcd.setCursor(8,1);			// auch nur dann den Cursor setzen (spart Zeit)
						lcd.write(0xE0); 			// Impuls-Symbol
					}
					
					dcfImpuls = false;
					dcfImpulsIsOn = true;
				} 
				else
				{
					lcd.setCursor(8,1);
					lcd.write(0x20); 
									
					dcfImpuls = false;
					dcfImpulsIsOn = false;
					
					// wenn Impulsdauer laenger an als langer Impuls -> Impulspause -> Impulsanzeige aus
					if(millis() > (dcfImpulsDauer + (2 * DCF_split_millis)))	
					{
						dcfImpulsIsOn = true;
						dcfImpulsDauer = millis();
					}
				}
			}
		}

		showAntenne();								// DCF77-Antenne anzeigen (falls enabled)
		//} DCF77-Impuls visualisieren ENDE

// ------------------------------------------------------------------------------------------------------------------
		//{ Anzeige der DCF77-Zeit
// ------------------------------------------------------------------------------------------------------------------
		if((bDCF77 == true) && (bRTCtime == false))
		{
			if(bDcfParity == true)								// Wenn beim Empfang der Zeit Parity ok war 
			{
				ucAntenneState = 2;								// animierte Ringe auf Antenne -> Suche
			}
			if((bDcfParity == true) && (bDcfTimeOk == true))	// Wenn Zeit valide ist
			{
				ucAntenneState = 3;								// statischer Ring auf Antenne
			}
			
			aktUhr = laufendeUhrzeit.now();

			if (bDcfTimeOk == true) rtcDS3231.adjust(aktUhr);	// wenn DCF korrekt erkannt -> RTC setzen, 2022-12-30

			// (year != 0) unterdrueckt das Datum, solange die Zeit noch nicht gesetzt ist
			if(((ucAktSekunde % 10) == 3) && (year != 0)) {		// Datum alle 10 s anzeigen (bei Sekunde == x3)
				lcd.setCursor(0, 0);
				// !!! diese Variante NICHT verwenden, braucht mehr Flash und ist langsamer !!!
				// lcd.print(day < 10 ? "0" : "");				// einstellige Tage mit fuehrender Null
				if(day < 10) lcd.print(F("0"));					// einstellige Tage mit fuehrender Null
				lcd.print(day);
				lcd.print(F("."));
				// lcd.print(mon < 10 ? "0" : "");				// einstellige Monate mit fuehrender Null
				if(mon < 10) lcd.print(F("0"));					// einstellige Monate mit fuehrender Null
				lcd.print(mon);
				lcd.print(F("."));
				lcd.print(year);
			} else {											// Zeit
				lcd.setCursor(0, 0);
				lcd.print(F(" "));								// erste Stelle des Datums loeschen
				// lcd.print(hh < 10 ? " " : "");				// einstellige Stunden mit fuehrender Null
				if(hh < 10) lcd.print(F(" "));					// einstellige Stunden einstellig anzeigen
				lcd.print(hh);
				lcd.print(F(":"));
				// lcd.print(mm < 10 ? "0" : "");				// einstellige Minuten mit fuehrender Null
				if(mm < 10) lcd.print(F("0"));					// einstellige Minuten mit fuehrender Null
				lcd.print(mm);
				lcd.print(F(":"));
				// lcd.print(ucAktSekunde < 10 ? "0" : "");		// einstellige Sekunden mit fuehrender Null
				if(ucAktSekunde < 10) lcd.print(F("0"));
				lcd.print(ucAktSekunde);
				lcd.print(F(" "));								// letzte Stelle des Datums loeschen
			}
		}
		//} Anzeige der DCF77-Zeit ENDE
		
// ------------------------------------------------------------------------------------------------------------------
		//{ Anzeige der RTClock-Zeit
// ------------------------------------------------------------------------------------------------------------------
		if((bRTCtime == true) && (bDCF77 == false))
		{
			aktUhr = rtcDS3231.now();							// aktuelle Zeit aus RTC auslesen
			if(aktUhr.hour() <= 24)								// RTC Modul wurde erkannt (ohne Modul kommt hier 165)
			{
				if((aktUhr.second() % 10) == 3)  {					// Datum alle 10 s anzeigen (bei Sekunde == x3)
					lcd.setCursor(0, 0);
					if(aktUhr.day() < 10) lcd.print(F("0"));		// einstellige Tage mit fuehrender Null
					lcd.print(aktUhr.day());
					lcd.print(F("."));
					if(aktUhr.month() < 10) lcd.print(F("0"));		// einstellige Monate mit fuehrender Null
					lcd.print(aktUhr.month());
					lcd.print(F("."));
					lcd.print(aktUhr.year());
				} else {										// Zeit
					lcd.setCursor(0, 0);
					lcd.print(F(" "));							// erste Stelle des Datums loeschen
					if(aktUhr.hour() < 10) lcd.print(F(" "));		// einstellige Stunden einstellig anzeigen
					lcd.print(aktUhr.hour());
					lcd.print(F(":"));
					if(aktUhr.minute() < 10) lcd.print(F("0"));	// einstellige Minuten mit fuehrender Null
					lcd.print(aktUhr.minute());
					lcd.print(F(":"));
					if(aktUhr.second() < 10) lcd.print(F("0"));
					lcd.print(aktUhr.second());
					lcd.print(F(" "));							// letzte Stelle des Datums loeschen
				}
			}
			else
			{													// RTC Modul wurde nicht erkannt
				lcd.setCursor(0, 0);
				lcd.print(F("no RTC... "));
			}
		}
		
		// aktuelle Uhrzeit in Minuten nach Mitternacht
		iaktZeitMinAbMidnight = (aktUhr.hour() * 60) + aktUhr.minute();
		// Serial.print(aktUhr.dayOfTheWeek());
		//} Anzeige der RTClock-Zeit ENDE

// ------------------------------------------------------------------------------------------------------------------
	
// ------------------------------------------------------------------------------------------------------------------
// Anzeige der Statemachine States 
// ------------------------------------------------------------------------------------------------------------------

		if((bStateDispOn == true) && (millis() > (uStateAnzeige + 120))) 
		{
			showStateAnzeige(false, 0xFF /*alle*/);	// Stateanzeige loeschen
		}

		// wenn Wandlung in den Sensoren gestartet wurde und Pause um ist -> Temp auslesen
		if((bConversionFlag == true) && (millis() > uConversionTimer + 750))	// Minimum ist 750ms
		{
#ifndef NOSENSORS
			getTemp();
#endif
			bConversionFlag = false;				// Wandlung ist abgeschlossen
		}

// ------------------------------------------------------------------------------------------------------------------
// Statemachine fuer die Steuerung
// ------------------------------------------------------------------------------------------------------------------
		switch(cGlobalState)
		{
#ifdef DEBUGON
			Serial.print(F("Statemachine\n"));
#endif
			case 1:									// State 1: Luefter ist aus, warten auf Ist-Temperatur > Soll-Temperatur
				//{ State anzeigen
				if(bState1FromManu == true)			// "manu" beendet, wieder "auto"
				{
					// eventuell laufenden Countdown fuer Check Aussentemp stoppen

					lcd.setCursor(0, 1);			// "auto" anzeigen
					lcd.print(F("auto    "));

					if(millis() > (uState1FromManuTimer + 2000))	// nach 2 Sekunden normale Anzeige
					{
						bState1FromManu = false;
						uNextCheckTimer = millis();			// Timer fuer naechsten Check ruecksetzen
					}
				}
				else
				{				
					if(bStateDispOn == false) {
						showStateAnzeige(true , cGlobalState);	// Anzeige des State mit kleinen Zahlen
						uStateAnzeige = millis();		// verschiedene Stati
					}
					showCountdown(-1);					// Countdown loeschen

					// Temperaturen pruefen
					bCoolWarm = (bCooling == true ? 
						(tempRaum > fSollTemp) : 
						(tempRaum < fSollTemp) );
					if(bCoolWarm)			// Raum muss gekuehlt/gewaermt werden
					{
						checkUhrzeit();					// pruefen, ob Luefter Vollgas erlaubt ist (uhrzeitabhaengig)
						setRelais(ON);					// Relais einschalten (nur wenn lt. Uhrzeit ok)
						uCheckVorlaufTimer = millis();	// Luefter ein fuer xx sec vor Messung aussen
						bShowLuefter = true;			// Lueftersymbol darstellen (in State 2 + 3)
						uCountdownTimer = millis();			// neuer Startwert
						iTmpChkVorlauf = iTempCheckVorlauf;	// Startwert laden
						cGlobalState = 2;				// naechster State: 2
					}
					else								// Raum muss nicht gekuehlt/gewaermt werden
					{
						setRelais(OFF);					// Relais ausschalten
						lcd.setCursor(9, 1);			// Lueftersymbol loeschen
						lcd.write(' ');					// Blank
						bShowLuefter = false;			// Lueftersymbol nicht darstellen (in State 2 + 3)
					}
				}
				break;	//}
				
			case 2:									// State 2: Luefter starten und eine Zeit lang laufen lassen
				//{ State anzeigen					//          um die Aussentemperatur im Rohr zu erfassen
				if(bStateDispOn == false) {
					showStateAnzeige(true , cGlobalState);	// Anzeige des State mit kleinen Zahlen
					uStateAnzeige = millis();
				}
				// Luefter anzeigen
				showLuefter();
				// Zeit pruefen
				if(millis() > (uCheckVorlaufTimer + (iTempCheckVorlauf * 1000)))	// Vorlaufzeit (xx s in ms!!) ist um
				{
					// Temperaturen pruefen
					bCoolWarm = (bCooling == true ? 
						(tempAussen < (tempRaum - iTempDiff)) : 
						(tempAussen > (tempRaum + iTempDiff)) );
					if(bCoolWarm)							// aussen kalt/warm genug?
					{										// ja
						// Luefter laufen lassen
						cGlobalState = 3;					// naechster State: 3	
						tempRaumEIN = tempRaum;
						uCountdownTimer = millis();			// neuer Startwert
					}
					else									// aussen zu warm
					{										// nein
						iTmpWaitNextChk = iWaitBeforeNextCheck;	// Startwert laden
						cGlobalState = 4;					// naechster State: 4					
						uNextCheckTimer = millis();			// Timer fuer naechsten Check starten
						uCountdownTimer = millis();			// neuer Startwert
					}
					iTmpChkVorlauf = iTempCheckVorlauf;		// fuer naechsten Durchgang laden
				}
				else										// Vorlaufzeit laeuft noch -> Countdown anzeigen
				{
					if(millis() > (uCountdownTimer + 980))	// 1 Sekunde vergangen?
					{
						uCountdownTimer = millis();			// neuer Startwert
						showCountdown(iTmpChkVorlauf--);
					}
				}
				break;	//}
				
			case 3:									// State 3: Aussen ist kuehler als Innen, Luefter laufen lassen
				//{ State anzeigen
				if(bStateDispOn == false) {					// State anzeigen
					showStateAnzeige(true , cGlobalState);	// Anzeige des State mit kleinen Zahlen
					uStateAnzeige = millis();
				}

				checkUhrzeit();					// pruefen, ob Luefter Vollgas erlaubt ist (uhrzeitabhaengig)
				setRelais(ON);					// Relais einschalten (nur wenn lt. Uhrzeit ok)
				showCountdown(-1);				// Countdown loeschen
				showLuefter();					// Luefter anzeigen
				
				// Temperaturen pruefen
				// (Innen Hysterese erreicht?) ODER (Aussen zu warm/kalt?)
				bCoolWarm = (bCooling == true ? 
					((tempRaum < (tempRaumEIN - iTempHyst)) || (tempAussen > (tempRaum - iTempDiff))) : 
					((tempRaum > (tempRaumEIN + iTempHyst)) || (tempAussen < (tempRaum + iTempDiff))) );
				if(bCoolWarm)
				{
					cGlobalState = 1;						// naechster State: 1
				}				
				break;	//}
				
			case 4:											// State 4: Aussen ist zu warm, Wartezeit starten, danach wieder zu State1
				//{ State anzeigen
				if(bStateDispOn == false) {
					showStateAnzeige(true , cGlobalState);	// Anzeige des State mit kleinen Zahlen
					uStateAnzeige = millis();
				}
				
				setRelais(OFF);								// Relais ausschalten
				// digitalWrite(RELAIS, LOW);				// Relais ausschalten
				lcd.setCursor(9, 1);						// Lueftersymbol loeschen
				lcd.write(' ');								// Blank
				
															// warte Wartezeit (10..30 min in ms!!) ab jetzt
				if(millis() > (uNextCheckTimer + ((unsigned long)iWaitBeforeNextCheck * 1000ul)))	
				{											// wenn Wartezeit um ist, erneut pruefen, ob innen noch zu warm
					cGlobalState = 1;						// naechster State: 1
				}
				else										// Wartezeit laeuft noch -> Countdown anzeigen
				{
					if(millis() > (uCountdownTimer + 980))	// 1 Sekunde vergangen?
					{
						uCountdownTimer = millis();			// neuer Startwert
						showCountdown(iTmpWaitNextChk--);
					}
				}
				break;	//}
				
			case 5:											// State 5, manueller Betrieb des Luefters
				//{ Relais einschalten
				bLuefterErlaubt = true;						// manuell muss immer gehen
				setRelais(ON);								// Relais einschalten
				bShowLuefter = true;						// Lueftersymbol darstellen (in State 5)
				showLuefter();
				break;	//}
				
			case 6:											// State 6, manueller Stopp des Luefters
				//{ Relais ausschalten
				setRelais(OFF);								// Relais ausschalten
				bShowLuefter = false;						// Lueftersymbol nicht darstellen (in State 6)
				showLuefter();
				break;	//}
				
			default:										// alle nicht definierten States muenden in State 1
				//{ State anzeigen
				if(bStateDispOn == false) {
					showStateAnzeige(true , cGlobalState);	// Anzeige des State mit kleinen Zahlen
					uStateAnzeige = millis();
				}
				cGlobalState = 1;							// naechster State: 1
				break;	//}
		}

// ------------------------------------------------------------------------------------------------------------------
// ab hier ausserhalb der Statemachine
// ------------------------------------------------------------------------------------------------------------------
		// DCF77-Empfang pruefen
		if(millis() > (ulAntenneTimer + 3000)) 				// wenn 3s rum, "kein DCF-Empfang" anzeigen
		{
			ulAntenneTimer = millis();
			ucAntenneState = 0; 							// keine DCF-Signale empfangen
			bDcfParity = false;								// DCF-Zeit invalide
			bDcfTimeOk = false;	

			if(bStateOnInInit == true)						// Impuls/Blank nur ausgeben, wenn im Init-Screen
			{
				lcd.setCursor(8,1);
				lcd.write(' '); 
			}
		}
		
		// Init-Screen anzeigen
		lcd.setCursor(0, 0);
		// kein DCF77 und keine RTC oder _beide_ aktiviert -> permanent Text "Temperatur" anzeigen
		if(((bDCF77 == false) && (bRTCtime == false)) || ((bDCF77 == true) && (bRTCtime == true))) 
		{		
			lcd.print(F("Temperatur"));
		}
		lcd.setCursor(1, 2);
		lcd.print(F("In:"));
		lcd.setCursor(0, 3);
		lcd.print(F("Out:"));

		bStateOnInInit = true;			// State nur im Initscreen anzeigen
		
		lcd.setCursor(4, 2);
		lcd.print(tempRaumTxt);
		lcd.setCursor(4, 3);
		lcd.print(tempAussenTxt);	

		if((GETKEYMACRO(QTUP)) &&  (bKeyUpTrig == false))
		{
#ifdef DEBUGTASTEN
			Serial.print(F("QTUP\n"));
#endif
			// Luefter manuell starten -> State 5 der Statemachine
			bStateDispOn = false;				// verhindern, dass "manu" sofort wieder geloescht wird
			lcd.setCursor(0, 1);
			lcd.print(F("manu ein"));
			cGlobalState = 5;
		}		
		else if((bKeyUpTrig == true) && (!GETKEYMACRO(QTUP)))	
		{										// bevor Taste erneut erkannt wird, muss Taste erst losgelassen werden
			bKeyUpTrig = false;					// Trigger loeschen
		}
		
		if((GETKEYMACRO(QTDOWN)) &&  (bKeyDownTrig == false))
		{
#ifdef DEBUGTASTEN
			Serial.print(F("QTDOWN\n"));
#endif
			// Luefter manuell stoppen -> State 6 der Statemachine -> Luefter stop
			lcd.setCursor(0, 1);
			lcd.print(F("manu aus"));
			cGlobalState = 6;
		}
		else if((bKeyDownTrig == true) && (!GETKEYMACRO(QTDOWN)))	
		{										// bevor Taste erneut erkannt wird, muss Taste erst losgelassen werden
			bKeyDownTrig = false;				// Trigger loeschen
		}
		
		if((GETKEYMACRO(QTLEFT)) &&  (bKeyLeftTrig == false))
		{
#ifdef DEBUGTASTEN
			Serial.print(F("QTLEFT\n"));
#endif			// Luefter manuell stoppen -> State 1 der Statemachine -> Temperaturen messen
			// lcd.setCursor(0, 1);				// Ausgabe des Textes "auto" erfolgt in State1
			// lcd.print(F("auto    "));
			cGlobalState = 1;
			bState1FromManu = true;				// beim Uebergang in State1 wird "auto" kurz angezeigt
			uState1FromManuTimer = millis();	// Timer fuer Anzeige von "auto" starten
		}
		else if((bKeyLeftTrig == true) && (!GETKEYMACRO(QTLEFT)))	
		{										// bevor Taste erneut erkannt wird, muss Taste erst losgelassen werden
			bKeyLeftTrig = false;				// Trigger loeschen
		}
		
		
	}	// \nicht im Menue
	else													// if(!inMenu)
	{
// ab hier _im_ Menue ---------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------------------
//{ mit der ENTER-Taste ins Menue
// ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTENTER)) && (bFirstTimeInMenu == true))		
	{
#ifdef DEBUGTASTEN
		Serial.print(F("QTENTER\n"));
#endif
		cChangePos = 0;								// beim Eintritt ins Menue einmalig Anleitung zeigen
		bFirstTimeInMenu = false;
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
		
		// alle aktuellen Parameter in Temporaervariablen speichern. Diese werden ggf. veraendert, bei ENTER uebernommen, bei ESC verworfen
		fTempMaxRoomtempval = fSollTemp;			// alte Werte fuer Edit merken
		iTempDifftempval 	= iTempDiff;	
		iTmpHysterese 		= iTempHyst;		
		iTmpVorlauf 		= iTempCheckVorlauf;		
		iTmpPause			= iWaitBeforeNextCheck;			
		iTmpInterval 		= iTempCheckInterval;	
		bTmpDCF77			= bDCF77;
		bTmpRTCtime			= bRTCtime;
		bTmpLuefterMitUhrSteuern = bLuefterMitUhrSteuern;
		cTmpSensorPos		= cSensorPos;
		
		while(GETKEYMACRO(QTENTER)) {};				// hier warten, bis ENTERKEY wieder losgelassen wurde
		
		lcd.clear();								// Anleitung
		lcd.setCursor(1, 0);
		lcd.print(F("Settings"));
		lcd.setCursor(0, 1);
		lcd.print(F("Params"));
		lcd.setCursor(1, 2);
		lcd.print(F("select:<>"));
		lcd.setCursor(1, 3);
		lcd.print(F("change:"));
		lcd.write(0xD0);							// Hoch-/Runter-Pfeil
		
		
	}
//}

// mit der ENTER-Taste ins Menue gekommen, ab hier wird mit RECHTS/LINKS zwischen den einzelnen	Parametern gewechselt
// ----------------------------------------------------------------------------------------------------------------------------
//{ mit RIGHT/LEFT die verschiedenen Parameter ansteuern, mit RIGHT lange Spezial-Fkt. aktivieren (noch ungenutzt)
// ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTRIGHT)) && (bKeyRightTrig == false))
	{
#ifdef DEBUGTASTEN
		Serial.print(F("QTRIGHT\n"));
#endif
		lcd.clear();
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden

		// wenn Wochentag- oder Wochend-Block aktiv ist, bleiben wir innerhalb der Zeiteinstellung
		if((bmdmdfBlockSel == true) || (bssBlockSel == true))
		{
			if(cChangePos < 12) cChangePos++; else cChangePos = 7;
		}
		else		// keiner der beiden Bloecke aktiv
		{
			if(cChangePos == 8) cChangePos = 13;	// Uhrzeiten ueberspringen
			else
				if(cChangePos < CHANGEPOSMAX) cChangePos++; else cChangePos = 1;
		}
		bKeyRightTrig = true;						// merken, dass getriggert wurde
	}
	else if((bKeyRightTrig == true) && (!GETKEYMACRO(QTRIGHT)))
	{
		bKeyRightTrig = false;						// Trigger loeschen
	}

/*
	if((GETKEYMACRO(QTRIGHT)) && (bKeyRightTrig == false) && (bLongKeyRight == false))
	{
		lcd.clear();
		lRightKeyPressed = millis();				// Startzeit des Tastendrucks merken (fuer LongRight)
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
													
		if(cChangePos < CHANGEPOSMAX) cChangePos++; else cChangePos = 1;
		
		// fuer den Fall, dass wir das benoetigen: langen Tastendruck RIGHT auswerten und speichern
		while((GETKEYMACRO(QTRIGHT)) && (bLongKeyRight == false))	
		{	
			if(millis() > (lRightKeyPressed + 1000))// RIGHTKEY laenger als 1s gedrueckt?
			{										
				bLongKeyRight = true;				//  -> "kontinuierlich" fuer Gesamt-Tassenzahl einschalten
			}
		}		
		bKeyRightTrig = true;						// merken, dass getriggert wurde
	}
	else if((bKeyRightTrig == true) && (!GETKEYMACRO(QTRIGHT)))
	{
		bKeyRightTrig = false;						// Trigger loeschen
	}
*/	
// ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTLEFT)) && (bKeyLeftTrig == false))
	{
#ifdef DEBUGTASTEN
		Serial.print(F("QTLEFT\n"));
#endif
		lcd.clear();
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden

		// wenn Wochentag- oder Wochend-Block aktiv ist, bleiben wir innerhalb der Zeiteinstellung
		if((bmdmdfBlockSel == true) || (bssBlockSel == true))
		{
			if(cChangePos > 7) cChangePos--; else cChangePos = 12;
		}
		else		// keiner der beiden Bloecke aktiv
		{
			if(cChangePos == 13) cChangePos = 8;	// Uhrzeiten ueberspringen
			else
				if(cChangePos > 1) cChangePos--; else cChangePos = CHANGEPOSMAX;
		}
		bKeyLeftTrig = true;						// merken, dass getriggert wurde
	}
	else if((bKeyLeftTrig == true) && (!GETKEYMACRO(QTLEFT)))
	{
		bKeyLeftTrig = false;						// Trigger loeschen
	}
//}

// ----------------------------------------------------------------------------------------------------------------------------
//{ Status "RIGHTKEY lange gedrueckt" wieder abschalten
// ----------------------------------------------------------------------------------------------------------------------------
	// if((GETKEYMACRO(QTRIGHT)) && (bKeyRightTrig == false) && (bLongKeyRight == true))
	// {
		// lRightKeyLongPressed = millis();									// Startzeit des Tastendrucks merken
		// lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// // der Timer muss bei jedem Tastendruck nachgetriggert werden
		// while((GETKEYMACRO(QTRIGHT)) && (bLongKeyRight == true))			// RIGHT laenger als 1s gedrueckt?
		// {																	//  -> "kontinuierlich" fuer Gesamt-Tassenzahl ausschalten
			// if(millis() > (lRightKeyLongPressed + 1000))
			// {
				// bLongKeyRight = false;
			// }
		// }		
		// bKeyRightTrig = true;												// merken, dass getriggert wurde
	// }
	// else if((bKeyRightTrig == true) && (!GETKEYMACRO(QTRIGHT)) && (bLongKeyRight == true))
	// {
		// bKeyRightTrig = false;												// Trigger loeschen
	// }
//}


// ----------------------------------------------------------------------------------------------------------------------------
// ab hier werden die Reaktionen der verschiedenen Parameter auf Tasten UP/DOWN ausprogrammiert
// abhaengig von cChangePos
// ----------------------------------------------------------------------------------------------------------------------------
// Zuordnung zwischen cChangePos und Parameter ist wie folgt:
// cChangePos == 1	"Temp"		"Schwelle"							[nicht begrenzt]
// cChangePos == 2   			"TempDiff"							[0..9]
// cChangePos == 3   			"Hysterese"							[1..9]
// cChangePos == 4  "Zeiten"	"Vorlauf"							[10..60]
// cChangePos == 5				"Pause"								[10..3600]
// cChangePos == 6				"Intervall"							[1..60]
// cChangePos == 7				"Laufzeit"	"mdmdf"	(Block Mo..Fr, [selektiert "MDMDF"/nicht selektiert "mdmdf"])
// cChangePos == 8							"ss"	(Block Sa+So, [selektiert "SS"/nicht selektiert "ss"])
// cChangePos == 9							"Start Stunde"			[0..23]
// cChangePos == 10							"Start Minute"			[0..59]
// cChangePos == 11							"Ende  Stunde"			[0..23]
// cChangePos == 12							"Ende  Minute"			[0..59]
// cChangePos == 13	"Modus"  										[kühlen/wärmen]
// cChangePos == 14	"Settings"	"DCF77"								[Ein/Aus]
// cChangePos == 15				"RTClock"							[Ein/Aus]
// cChangePos == 16				"Version"
// cChangePos == 17				"Zeitgeführt"						[Ein/Aus]
// cChangePos == 18				"Sensor einlernen"					[Innen/Aussen/WR] (WR = Wechselrichter)
// (cChangePos == 19			"Defaults"							[ESC/OK])	weglassen?
// ----------------------------------------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------------------------------------------------------
//{ Taste UP
// ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTUP)) &&  (bKeyUpTrig == false))
	{
		bKeyUpTrig = true;					// Flag fuer SingleStep
		// lUpKeyPressed = millis();		// Startzeit des Tastendrucks merken (fuer LongUp) UNVOLLSTAENDIG!!
		lMenuTimeout = millis();			// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
											// der Timer muss bei jedem Tastendruck nachgetriggert werden
											
		if(cChangePos == 1)					// Temperatur ab der Luefter laufen soll (Schwelle) [nicht begrenzt]
		{
			fTempMaxRoomtempval += 0.1;
		}
		else if(cChangePos == 2)			// minimale TempDiff zw. Innen und Aussen, damit Luefter laeuft [0..9]
		{
			if(iTempDifftempval < 9) iTempDifftempval++;		// maximal 9 Grad Differenz einstellbar
		}
		else if(cChangePos == 3)			// Hysterese zum Abschalten 			[1..9]
		{
			if(iTmpHysterese < 9) iTmpHysterese++;		// maximal 9K Hysterese einstellbar
		}
		else if(cChangePos == 4)			// Vorlaufzeit, bevor Aussentemperatur gemessen wird [10..60]
		{
			if(iTmpVorlauf < 60) iTmpVorlauf++;		// maximal 60s Vorlauf einstellbar
		}
		else if(cChangePos == 5)			// Pause, vor naechster Messung wenn aussen zu warm [10..3600]
		{
			if(iTmpPause < 3600) iTmpPause += 10;	// maximal 3600s (60 min) Vorlauf einstellbar
		}
		else if(cChangePos == 6)			// Intervall fuer Temperaturmessungen 	[1..60]
		{
			if(iTmpInterval < 60) iTmpInterval++;	// maximal 60s Intervall einstellbar
		}
		
		// Start - Laufzeiten des Luefters einstellen
		else if(cChangePos == 7)			// Laufzeit, Block Mo..Fr, [selektiert "MDMDF"/nicht selektiert "mdmdf"]
		{
			if(bmdmdfBlockSel == true) bmdmdfBlockSel = false; 
			else {bmdmdfBlockSel = true; 	// immer toggeln
				  bssBlockSel = false;		// immer nur ein TageBlock aktiv
			}
			binTimerScreen = false;
		}
		else if(cChangePos == 8)			// Laufzeit, Block Sa+So, [selektiert "SS"/nicht selektiert "ss"]
		{
			if(bssBlockSel == true) bssBlockSel = false; 
			else {bssBlockSel = true;		// immer toggeln
				  bmdmdfBlockSel = false;	// immer nur ein TageBlock aktiv
			}
			binTimerScreen = false;
		}
		
		// wenn Wochentag- oder Wochend-Block aktiv ist, bleiben wir innerhalb der Zeiteinstellung
		if((bmdmdfBlockSel == true) || (bssBlockSel == true))
		{
			if(cChangePos == 9)			// Laufzeit, "Start Stunde"				[0..23]
			{
				if(bmdmdfBlockSel == true) {if(WoTage.StartStunde < 23) WoTage.StartStunde++;}
				if(bssBlockSel == true)    {if(WoEnde.StartStunde < 23) WoEnde.StartStunde++;}
				binTimerScreen = false;
			}
			else if(cChangePos == 10)			// Laufzeit, "Start Minute"				[0..59]
			{
				if(bmdmdfBlockSel == true) {if(WoTage.StartMinute < 59) WoTage.StartMinute++;}
				if(bssBlockSel == true)    {if(WoEnde.StartMinute < 59) WoEnde.StartMinute++;}
				binTimerScreen = false;
			}
			else if(cChangePos == 11)			// Laufzeit, "Ende  Stunde"				[0..23]
			{
				if(bmdmdfBlockSel == true) {if(WoTage.EndeStunde < 23) WoTage.EndeStunde++;}
				if(bssBlockSel == true)    {if(WoEnde.EndeStunde < 23) WoEnde.EndeStunde++;}
				binTimerScreen = false;
			}
			else if(cChangePos == 12)			// Laufzeit, "Ende  Minute"				[0..59]
			{
				if(bmdmdfBlockSel == true) {if(WoTage.EndeMinute < 59) WoTage.EndeMinute++;}
				if(bssBlockSel == true)    {if(WoEnde.EndeMinute < 59) WoEnde.EndeMinute++;}
				binTimerScreen = false;
			}
		}
		// Ende - Laufzeiten des Luefters einstellen
		
		else if(cChangePos == 13)			// Modus 								[kühlen/wärmen]
		{
			if(bTmpCooling == true) bTmpCooling = false; else bTmpCooling = true;	// immer toggle
		}
		else if(cChangePos == 14)			// DCF77 								[Ein/Aus]
		{
			if(bTmpDCF77 == true) bTmpDCF77 = false; else bTmpDCF77 = true;			// immer toggle
		}
		else if(cChangePos == 15)			// RTC									[Ein/Aus]
		{
			if(bTmpRTCtime == true) bTmpRTCtime = false; else bTmpRTCtime = true;	// immer toggle
		}
		else if(cChangePos == 16)			// Versionen
		{			
		}
		else if(cChangePos == 17)			// zeitgefuehrt							[Ein/Aus]
		{
			if(bTmpLuefterMitUhrSteuern == true) bTmpLuefterMitUhrSteuern = false; 
				else bTmpLuefterMitUhrSteuern = true;								// immer toggle
		}
		else if(cChangePos == 18)			// Sensor einlernen						[Innen/Aussen/WR]
		{			
			if(cTmpSensorPos < 3) cTmpSensorPos++; else cTmpSensorPos = 1;					// 1..3
		}
	}		
	else if((bKeyUpTrig == true) && (!GETKEYMACRO(QTUP)))	
	{										// bevor Taste erneut erkannt wird, muss Taste erst losgelassen werden
		bKeyUpTrig = false;					// Trigger loeschen
	}
//}

// ----------------------------------------------------------------------------------------------------------------------------
//{ Taste DOWN
// ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTDOWN)) &&  (bKeyDownTrig == false))
	{
		bKeyDownTrig = true;					// Flag fuer SingleStep
		lMenuTimeout = millis();			// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
											// der Timer muss bei jedem Tastendruck nachgetriggert werden
											
		if(cChangePos == 1)					// Temperatur ab der Luefter laufen soll (Schwelle) [nicht begrenzt]
		{
			fTempMaxRoomtempval -= 0.1;
		}
		else if(cChangePos == 2)			// minimale TempDiff zw. Innen und Aussen, damit Luefter laeuft [0..9]
		{
			if(iTempDifftempval > 0) iTempDifftempval--;		// minimal 0 Grad Differenz einstellbar
		}
		else if(cChangePos == 3)			// Hysterese zum Abschalten 			[1..9]
		{
			if(iTmpHysterese > 1) iTmpHysterese--;	// minimal 1K einstellbar
		}
		else if(cChangePos == 4)			// Vorlaufzeit, bevor Aussentemperatur gemessen wird [10..60]
		{
			if(iTmpVorlauf > 10) iTmpVorlauf--;		// minimal 10s einstellbar
		}
		else if(cChangePos == 5)			// Pause, vor naechster Messung wenn aussen zu warm [10..3600]
		{
			if(iTmpPause > 10) iTmpPause -= 10;		// minimal 10s einstellbar
		}
		else if(cChangePos == 6)			// Intervall fuer Temperaturmessungen 	[1..60]
		{
			if(iTmpInterval > 1) iTmpInterval--;	// minimal 1s einstellbar
		}
		
		// Start - Laufzeiten des Luefters einstellen
		else if(cChangePos == 7)			// Laufzeit, Block Mo..Fr, [selektiert "MDMDF"/nicht selektiert "mdmdf"]
		{
			if(bmdmdfBlockSel == true) bmdmdfBlockSel = false; 
			else {bmdmdfBlockSel = true; 	// immer toggeln
				  bssBlockSel = false;		// immer nur ein TageBlock aktiv
			}
			binTimerScreen = false;
		}
		else if(cChangePos == 8)			// Laufzeit, Block Sa+So, [selektiert "SS"/nicht selektiert "ss"]
		{
			if(bssBlockSel == true) bssBlockSel = false; 
			else {bssBlockSel = true;		// immer toggeln
				  bmdmdfBlockSel = false;	// immer nur ein TageBlock aktiv
			}
			binTimerScreen = false;
		}
		
		// wenn Wochentag- oder Wochend-Block aktiv ist, bleiben wir innerhalb der Zeiteinstellung
		if((bmdmdfBlockSel == true) || (bssBlockSel == true))
		{
			 if(cChangePos == 9)			// Laufzeit, "Start Stunde"				[0..23]
			{
				if(bmdmdfBlockSel == true) {if(WoTage.StartStunde > 0) WoTage.StartStunde--;}
				if(bssBlockSel == true)    {if(WoEnde.StartStunde > 0) WoEnde.StartStunde--;}
				binTimerScreen = false;
			}
			else if(cChangePos == 10)			// Laufzeit, "Start Minute"				[0..59]
			{
				if(bmdmdfBlockSel == true) {if(WoTage.StartMinute > 0) WoTage.StartMinute--;}
				if(bssBlockSel == true)    {if(WoEnde.StartMinute > 0) WoEnde.StartMinute--;}
				binTimerScreen = false;
			}
			else if(cChangePos == 11)			// Laufzeit, "Ende  Stunde"				[0..23]
			{
				if(bmdmdfBlockSel == true) {if(WoTage.EndeStunde > 0) WoTage.EndeStunde--;}
				if(bssBlockSel == true)    {if(WoEnde.EndeStunde > 0) WoEnde.EndeStunde--;}
				binTimerScreen = false;
			}
			else if(cChangePos == 12)			// Laufzeit, "Ende  Minute"				[0..59]
			{
				if(bmdmdfBlockSel == true) {if(WoTage.EndeMinute > 0) WoTage.EndeMinute--;}
				if(bssBlockSel == true)    {if(WoEnde.EndeMinute > 0) WoEnde.EndeMinute--;}
				binTimerScreen = false;
			}
		}
		// Ende - Laufzeiten des Luefters einstellen
		
		else if(cChangePos == 13)			// Modus 								[kühlen/wärmen]
		{
			if(bTmpCooling == true) bTmpCooling = false; else bTmpCooling = true;	// immer toggle
		}
		else if(cChangePos == 14)			// DCF77 								[Ein/Aus]
		{
			if(bTmpDCF77 == true) bTmpDCF77 = false; else bTmpDCF77 = true;			// immer toggle
		}
		else if(cChangePos == 15)			// RTC									[Ein/Aus]
		{
			if(bTmpRTCtime == true) bTmpRTCtime = false; else bTmpRTCtime = true;	// immer toggle
		}
		else if(cChangePos == 16)			// Versionen
		{
		}
		else if(cChangePos == 17)			// zeitgefuehrt							[Ein/Aus]
		{
			if(bTmpLuefterMitUhrSteuern == true) bTmpLuefterMitUhrSteuern = false; 
				else bTmpLuefterMitUhrSteuern = true;								// immer toggle
		}
		else if(cChangePos == 18)			// Sensor einlernen						[Innen/Aussen/WR]
		{			
			if(cTmpSensorPos > 1) cTmpSensorPos--; else cTmpSensorPos = 3;					// 3..1
		}
	}		
	else if((bKeyDownTrig == true) && (!GETKEYMACRO(QTDOWN)))	
	{										// bevor Taste erneut erkannt wird, muss Taste erst losgelassen werden
		bKeyDownTrig = false;				// Trigger loeschen
	}
//}

// ----------------------------------------------------------------------------------------------------------------------------
//{ Taste ENTER
// ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTENTER)) &&  (bKeyEnterTrig == false))
	{
		bKeyEnterTrig = true;					// Flag fuer SingleStep
		lMenuTimeout = millis();			// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
											// der Timer muss bei jedem Tastendruck nachgetriggert werden
											
		if(cChangePos == 1)					// Temperatur ab der Luefter laufen soll (Schwelle) [nicht begrenzt]
		{
			fSollTemp = fTempMaxRoomtempval;	// neuen Wert uebernehmen
		}
		else if(cChangePos == 2)			// minimale TempDiff zw. Innen und Aussen, damit Luefter laeuft [0..9]
		{
			iTempDiff = iTempDifftempval;	// neuen Wert uebernehmen
		}
		else if(cChangePos == 3)			// Hysterese zum Abschalten 			[1..9]
		{
			iTempHyst = iTmpHysterese;		// neuen Wert uebernehmen
		}
		else if(cChangePos == 4)			// Vorlaufzeit, bevor Aussentemperatur gemessen wird [10..60]
		{
			iTempCheckVorlauf = iTmpVorlauf;	// neuen Wert uebernehmen
		}
		else if(cChangePos == 5)			// Pause, vor naechster Messung wenn aussen zu warm [10..3600]
		{
			iWaitBeforeNextCheck = iTmpPause;	// neuen Wert uebernehmen
		}
		else if(cChangePos == 6)			// Intervall fuer Temperaturmessungen 	[1..60]
		{
			iTempCheckInterval = iTmpInterval;	// neuen Wert uebernehmen
		}
		else if((cChangePos >= 7)  && (cChangePos <= 12))	// Laufzeit, Block Mo..Fr, [selektiert "MDMDF"/nicht selektiert "mdmdf"]
		{
			// Uhrzeiten in Minuten nach Mitternacht aus den einzelnen Stellen zusammen basteln
			WoTage.StartMinAbMidnight = (WoTage.StartStunde * 60) + WoTage.StartMinute;
			WoTage.EndeMinAbMidnight  = (WoTage.EndeStunde * 60) + WoTage.EndeMinute;
			
			WoEnde.StartMinAbMidnight = (WoEnde.StartStunde * 60) + WoEnde.StartMinute;
			WoEnde.EndeMinAbMidnight  = (WoEnde.EndeStunde * 60) + WoEnde.EndeMinute;
		}
		else if(cChangePos == 13)			// Modus 								[kühlen/wärmen]
		{
			bCooling = bTmpCooling;			// neuen Wert uebernehmen
			bState1FromManu = true;			// Trick um Countdown timer zu loeschen
		}
		else if(cChangePos == 14)			// DCF77 								[Ein/Aus]
		{
			bDCF77 = bTmpDCF77;
		}
		else if(cChangePos == 15)			// RTC									[Ein/Aus]
		{
			bRTCtime = bTmpRTCtime;			// neuen Wert uebernehmen
		}
		else if(cChangePos == 16)			// Versionen
		{
		}
		else if(cChangePos == 17)			// Versionen
		{
			bLuefterMitUhrSteuern = bTmpLuefterMitUhrSteuern;	// neuen Wert uebernehmen
		}
		else if(cChangePos == 18)			// Sensor einlernen
		{			
			switch (cTmpSensorPos)
			{
				case 1:						// innen
				break;
				case 2:						// aussen
				break;
				case 3:						// Wechselrichter
				break;
				default:
				break;
			}
		}
		
		inMenu = false;						// Menue wird verlassen

		// falls ein Parameter geaendert wurde, wird er jetzt im EEPROM gespeichert
		saveToEEPROM();	// prueft, ob der jeweilige Wert geaendert ist, falls ja -> neu speichern
	}
	else if((bKeyEnterTrig == true) && (!GETKEYMACRO(QTENTER)))	
	{										// bevor Taste erneut erkannt wird, muss Taste erst losgelassen werden
		bKeyEnterTrig = false;				// Trigger loeschen
	}
//}

// ----------------------------------------------------------------------------------------------------------------------------
//{ Hier werden die dynamischen Elemente der Texte im Menue dargestellt (abhaengig von cChangePos)
// ----------------------------------------------------------------------------------------------------------------------------
	if(cChangePos == 1)					// Temperatur ab der Luefter laufen soll (Schwelle) [nicht begrenzt]
	{
		char tempmaxRoomTxt[8] = "";	// fuer formatierte Ausgabe der Solltemperatur
		
		dtostrf(fTempMaxRoomtempval, 4, 1, tempmaxRoomTxt); // 4 ist Breite der Ausgabe inklusive aller Zeichen, 1 ist Nachkommastellen
		lcd.setCursor(2, 3); 			// set cursor
		lcd.print(tempmaxRoomTxt); 		// print change content

		lcd.setCursor(0, 0);
		lcd.print(F("Ab welcher"));  
		lcd.setCursor(0, 1);
		lcd.print(F("Temperatur"));
		lcd.setCursor(0, 2);
		lcd.print(F(" starten?"));

		lcd.setCursor(2, 3); 			// set cursor   
		lcd.print(fSollTemp); 			// print change content
		lcd.setCursor(6, 3); 			// set cursor   
		lcd.write(0xcc);				// Grad-Zeichen
		lcd.setCursor(7, 3);
		lcd.print(F("C"));
		valueChanged = false;		// globaler Merker, ob ins EEPROM gespeichert werden soll
	}
	else if(cChangePos == 2)			// minimale TempDiff zw. Innen und Aussen, damit Luefter laeuft [0..9]
	{
		lcd.setCursor(0, 0);
		lcd.print(F("Differenz"));  
		lcd.setCursor(0, 1);
		lcd.print(F(" innen/"));
		lcd.setCursor(0, 2);
		lcd.print(F(" aussen"));
		lcd.setCursor(5, 3);
		lcd.print(F("K"));			// Kelvin, da kein Absolutwert, sondern eine Differenz

		lcd.setCursor(4, 3); 		// set cursor   
		lcd.print(iTempDifftempval); // print change content
		valueChanged = false;		// globaler Merker, ob ins EEPROM gespeichert werden soll
	}
	else if(cChangePos == 3)		// Hysterese zum Abschalten 			[1..9]
	{
		lcd.setCursor(0, 0);
		lcd.print(F(" Schalt-"));  
		lcd.setCursor(0, 1);
		lcd.print(F("hysterese"));
		lcd.setCursor(5, 3);
		lcd.print(F("K"));			// Sekunden

		lcd.setCursor(4, 3); 		// set cursor   
		lcd.print(iTmpHysterese); 	// print change content
		valueChanged = false;		// globaler Merker, ob ins EEPROM gespeichert werden soll
	}
	else if(cChangePos == 4)			// Vorlaufzeit, bevor Aussentemperatur gemessen wird [10..60]
	{
		lcd.setCursor(0, 0);
		lcd.print(F(" Vorlauf"));  
		lcd.setCursor(0, 1);
		lcd.print(F(" Messung"));
		lcd.setCursor(0, 2);
		lcd.print(F(" aussen"));
		lcd.setCursor(6, 3);
		lcd.print(F("s"));			// Sekunden

		lcd.setCursor(4, 3); 		// set cursor   
		lcd.print(iTmpVorlauf); 	// print change content
		valueChanged = false;		// globaler Merker, ob ins EEPROM gespeichert werden soll
	}
	else if(cChangePos == 5)			// Pause, vor naechster Messung wenn aussen zu warm [10..3600]
	{
		lcd.setCursor(0, 0);
		lcd.print(F("Pause wenn"));  
		lcd.setCursor(0, 1);
		lcd.print(F("aussen zu"));
		lcd.setCursor(0, 2);
		if(bCooling == true) lcd.print(F("  warm")); else lcd.print(F("  kalt")); 
		lcd.setCursor(6, 3);
		lcd.print(F("s"));			// Sekunden

		lcd.setCursor(2, 3); 		// set cursor   
		if (iTmpPause < 1000) lcd.print(" "); 
		if (iTmpPause < 100) lcd.print(" "); 
		if (iTmpPause < 10)  lcd.print(" ");
		lcd.print(iTmpPause); 		// print change content
		valueChanged = false;		// globaler Merker, ob ins EEPROM gespeichert werden soll
	}
	else if(cChangePos == 6)			// Intervall fuer Temperaturmessungen 	[1..60]
	{
		lcd.setCursor(0, 0);
		lcd.print(F("Temperatur"));  
		lcd.setCursor(0, 1);
		lcd.print(F("  messen"));
		lcd.setCursor(0, 2);
		lcd.print(F("   alle"));
		lcd.setCursor(6, 3);
		lcd.print(F("s"));			// Sekunden

		lcd.setCursor(3, 3); 		// set cursor   
		if (iTmpInterval < 100) lcd.print(" "); 
		if (iTmpInterval < 10)  lcd.print(" ");
		lcd.print(iTmpInterval); 	// print change content
		valueChanged = false;		// globaler Merker, ob ins EEPROM gespeichert werden soll
	}
	else if(cChangePos == 7)		// Laufzeit, Block Mo..Fr, [selektiert "MDMDF"/nicht selektiert "mdmdf"]
	{
		showTimerScreen();
		lcd.setCursor(1, 1); 			// Cursor auf "mdmdf"
	}
	else if(cChangePos == 8)			// Laufzeit, Block Sa+So, [selektiert "SS"/nicht selektiert "ss"]
	{
		showTimerScreen();
		lcd.setCursor(6, 1); 			// Cursor auf "ss"
	}
	else if(cChangePos == 9)			// Laufzeit, "Start Stunde"				[0..23]
	{
		showTimerScreen();
		lcd.setCursor(4, 2); 			// Cursor auf "Start Stunde"	
	}
	else if(cChangePos == 10)			// Laufzeit, "Start Minute"				[0..59]
	{
		showTimerScreen();
		lcd.setCursor(7, 2); 			// Cursor auf "Start Minute"	
	}
	else if(cChangePos == 11)			// Laufzeit, "Ende  Stunde"				[0..23]
	{
		showTimerScreen();
		lcd.setCursor(4, 3); 			// Cursor auf "Ende  Stunde"	
	}
	else if(cChangePos == 12)			// Laufzeit, "Ende  Minute"				[0..59]
	{
		showTimerScreen();
		lcd.setCursor(7, 3); 			// Cursor auf "Ende  Minute"	
	}
	else if(cChangePos == 13)			// Modus 								[kühlen/wärmen]
	{
		lcd.setCursor(0, 0);
		lcd.print(F("  Modus"));  
		lcd.setCursor(0, 2);
		lcd.print(F("  Raum"));

		lcd.setCursor(2, 3); // set cursor   
		if(bTmpCooling == true) {
		lcd.print(F(" k"));  lcd.write(0x81); lcd.print(F("hlen"));}			// print "kühlen"
		else {lcd.print(F(" w")); lcd.write(0x84); lcd.print(F("rmen"));}		// print "wärmen"
	}
	else if(cChangePos == 14)			// DCF77 								[Ein/Aus]
	{
		lcd.setCursor(2, 0);
		lcd.print(F("DCF77"));  
		lcd.setCursor(2, 1);
		lcd.print(F("Zeit"));
		lcd.setCursor(0, 2);
		lcd.print(F("anzeigen?"));

		lcd.setCursor(3, 3); // set cursor   
		lcd.print((bTmpDCF77 == true) ? F(" Ja ") : F("Nein"));
	}
	else if(cChangePos == 15)			// RTC									[Ein/Aus]
	{
		lcd.setCursor(1, 0);
		lcd.print(F("RTClock"));  
		lcd.setCursor(2, 1);
		lcd.print(F("Zeit"));
		lcd.setCursor(0, 2);
		lcd.print(F("anzeigen?"));

		lcd.setCursor(3, 3); // set cursor   
		lcd.print((bTmpRTCtime == true) ? F(" Ja ") : F("Nein"));
	}
	else if(cChangePos == 16)			// Versionen
	{
		showVersion();
	}
	else if(cChangePos == 17)			// zeitgefuehrt							[Ein/Aus]
	{
		lcd.setCursor(0, 0);
		lcd.print(F("L")); lcd.write(0x81); lcd.print(F("fter nur"));  
		lcd.setCursor(1, 1);
		lcd.print(F("tags")); lcd.write(0x81); lcd.print(F("ber"));
		lcd.setCursor(1, 2);
		lcd.print(F("Vollgas?"));

		lcd.setCursor(3, 3); // set cursor   
		lcd.print((bTmpLuefterMitUhrSteuern == true) ? F(" Ja ") : F("Nein"));
	}
	else if(cChangePos == 18)			// Sensor einlernen
	{			
		lcd.setCursor(2, 0);
		lcd.print(F("Sensor")); 
		lcd.setCursor(0, 1);
		lcd.print(F("(Adresse)"));
		lcd.setCursor(2, 2);
		lcd.print((cTmpSensorPos == 1 ? F("Innen ") : (cTmpSensorPos == 2 ? F("Aussen") : F("  WR  "))));
		lcd.setCursor(1, 3);
		lcd.print(F("OK / ESC"));
	}
	
	if((cChangePos < 7) || (cChangePos > 12)) {
		binTimerScreen = false;
		lcd.noBlink();
		lcd.noCursor();
	}
//}

// ----------------------------------------------------------------------------------------------------------------------------
//{ Menue verlassen (hier muss keine Verriegelung programmiert werden)
// ----------------------------------------------------------------------------------------------------------------------------
  // Das Menu wird verlassen wenn ESC betaetigt wird oder wenn der Timeout abgelaufen ist, ohne dass 
  // eine Taste betaetigt wurde.
  	if((GETKEYMACRO(QTESC)) || (millis() > (lMenuTimeout + AUTOLEAVEMENU)))
	{
#ifdef DEBUGTASTEN
		Serial.print(F("QTESC\n"));
#endif
		inMenu = false;					// Menue verlassen
		bLongKeyRight = false;			// kontinuierlichen Modus loeschen
		
		// Werte im EEPROM permanent machen, nur geaenderte Werte werden dabei geschrieben
		saveToEEPROM();					
		
		bFirstTimeInMenu = true;		// beim Verlassen des Menu wieder setzen
		
		lcd.noCursor();					// Cursor abschalten, falls gerade Laufzeiten eingestellt wurden
		lcd.noBlink();					// Cursor abschalten, falls gerade Laufzeiten eingestellt wurden
		lcd.clear();
	}
//}
  }	// \im Menue
  
	// Aktivitaetsueberwachung. Hiermit ist feststellbar, ob die Kommunikation zum Display (I2C) 
	// zusammengebrochen ist, oder der µC haengt...
	if(millis() > (lAktivityBlinker + 1000)) 
	{
		digitalWrite(LED13, !digitalRead(LED13));		// LED13 blinkt im Sekundentakt
		lAktivityBlinker = millis();
	}
  
	// periodische Behandlung der DCF77-Decodierung
	if(bDCF77 == true)
	{
		if (ss != previousSecond) 
		{
			previousSecond = ss;
		}
		if (DCFSignalState != previousSignalState) 
		{
			scanSignal();
			previousSignalState = DCFSignalState;
		}
	}
	// per Menue ist die Frequenz der getTemp() Aufrufe einstellbar (alle 1..9 Sekunden)
	if(millis() > (llastTempGetTime + (iTempCheckInterval * 1000)))
	{
		// die Temperatursensoren DS18B20 benoetigen in der hoechsten Aufloesung ~750 ms fuer die Wandlung
		// also empfielt es sich, den Start der Wandlung und das Auslesen der Ergebnisse in getrennte
		// Routinen zu packen, so dass die Haupt-Loop nicht zyklisch durch warten auf die Ergebnisse 
		// ausgebremst wird
		if (bConversionFlag == false)
		{
#ifndef NOSENSORS
			startConversion();	// wenn Wandlung noch nicht laeuft -> starten	
#endif
		}
		// -->> das Abholen der Werte erfolgt in der Hauptloop <<--
		
		llastTempGetTime = millis();
	}
}





// -----------------------------------------------------------------------
// subroutines
// -----------------------------------------------------------------------
// Wandlung der Temperatursensoren starten
void startConversion()
{
	TempR.reset();				// Raumtemperatur messen
	TempR.select(addrR);
	TempR.write(0x44,1);         // start Konvertierung, mit power-on am Ende

	TempA.reset();				// Aussentemperatur messen
	TempA.select(addrA);
	TempA.write(0x44,1);         // start Konvertierung, mit power-on am Ende
	
	bConversionFlag = true;
	uConversionTimer = millis();	// Start conversion merken
}

// -----------------------------------------------------------------------
// Temperaturen auslesen
// -----------------------------------------------------------------------
void getTemp()
{
	// Serial.println(F("getTemp() gestartet"));
	
  // Raumtemperatur -------------------
  present = TempR.reset();
  TempR.select(addrR);    
  TempR.write(0xBE);         			// Wert lesen

  for ( i = 0; i < 9; i++) {           	// 9 bytes
    data[i] = TempR.read();
  }
	// Convert the data to actual temperature
	// because the result is a 16 bit signed integer, it should
	// be stored to an "int16_t" type, which is always 16 bits
	// even when compiled on a 32 bit processor.
	raw = (data[1] << 8) | data[0];

	cfg = (data[4] & 0x60);
	// at lower res, the low bits are undefined, so let's zero them
	if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
	else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
	else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
	//// default is 12 bit resolution, 750 ms conversion time

	tempRaum = (float)raw / 16.0;
	
	// tempRaum = -4.33;
	// tempRaum = tempRaum - 25.0;
	memcpy(tempbuff, "1", 2);							// Puffer loeschen
	if(tempRaum < 0) {
		tempRaumAbs = abs(tempRaum);					// falls negativ -> nur Absolutwert wandeln...
		tempbuff[0] = '-';
	} else {
		tempRaumAbs = tempRaum;							// ... sonst uebernehmen...
		tempbuff[0] = ' ';
	}
    dtostrf(tempRaumAbs, 4, 1, tempRaumTxt);  			// 4 ist Breite der Ausgabe inklusive aller Zeichen, 1 ist Nachkommastellen
	strcat(tempbuff, tempRaumTxt);						// ...und Vorzeichen manuell dranbasteln
	memcpy(tempRaumTxt, tempbuff, sizeof tempbuff);
	strcat(tempRaumTxt, "x");							// eine weitere Stelle dranhaengen (fuer °)
	tempRaumTxt[5] = 0xCC;
	
  // Aussentemperatur -------------------
  present = TempA.reset();
  TempA.select(addrA);    
  TempA.write(0xBE);         			// Wert lesen

  for ( i = 0; i < 9; i++) {           	// 9 bytes
    data[i] = TempA.read();
  }
	// Convert the data to actual temperature
	// because the result is a 16 bit signed integer, it should
	// be stored to an "int16_t" type, which is always 16 bits
	// even when compiled on a 32 bit processor.
	raw = (data[1] << 8) | data[0];

	cfg = (data[4] & 0x60);
	// at lower res, the low bits are undefined, so let's zero them
	if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
	else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
	else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
	//// default is 12 bit resolution, 750 ms conversion time

	tempAussen = (float)raw / 16.0;
	
	// tempAussen = -2.33;
	// tempAussen = tempAussen - 25.0;
	memcpy(tempbuff, "1", 2);							// Puffer loeschen
	if(tempAussen < 0) {
		tempAussenAbs = abs(tempAussen);		// falls negativ -> nur Absolutwert wandeln...
		tempbuff[0] = '-';
	} else {
		tempAussenAbs = tempAussen;				// ... sonst uebernehmen...
		tempbuff[0] = ' ';
	}
    dtostrf(tempAussenAbs, 4, 1, tempAussenTxt);  			// 4 ist Breite der Ausgabe inklusive aller Zeichen, 1 ist Nachkommastellen
	strcat(tempbuff, tempAussenTxt);						// ...und Vorzeichen manuell dranbasteln
	memcpy(tempAussenTxt, tempbuff, sizeof tempbuff);
	strcat(tempAussenTxt, "x");							// eine weitere Stelle dranhaengen (fuer °)
	tempAussenTxt[5] = 0xCC;
}

// -----------------------------------------------------------------------
// OneWire Anschlüsse initialisieren, Sensoren erkennen
// -----------------------------------------------------------------------
void initOneWireR()
{
  // Raumtemperatur
  if ( !TempR.search(addrR)) {
      Serial.print(F("Keine weiteren Adressen.\n"));
      TempR.reset_search();
      return;
  }

  Serial.print(F("R="));
  for( i = 0; i < 8; i++) {
    Serial.print(addrR[i], HEX);
    Serial.print(F(" "));
  }

  if ( OneWire::crc8( addrR, 7) != addrR[7]) {
      Serial.print(F("CRC ungueltig!\n"));
      return;
  }

  if ( addrR[0] == 0x10) {
      Serial.print(F("DS18S20 Familie.\n"));
  }
  else if ( addrR[0] == 0x28) {
      Serial.print(F("DS18B20 Familie.\n"));
  }
  else {
      Serial.print(F("Geraetefamilie nicht erkannt : 0x"));
      Serial.println(addrR[0],HEX);
      return;
  }
}
void initOneWireA()
{
  // Aussentemperatur
  if ( !TempA.search(addrA)) {
      Serial.print(F("Keine weiteren Adressen.\n"));
      TempA.reset_search();
      return;
  }

  Serial.print(F("R="));
  for( i = 0; i < 8; i++) {
    Serial.print(addrA[i], HEX);
    Serial.print(F(" "));
  }

  if ( OneWire::crc8( addrA, 7) != addrA[7]) {
      Serial.print(F("CRC ungueltig!\n"));
      return;
  }

  if ( addrA[0] == 0x10) {
      Serial.print(F("DS18S20 Familie.\n"));
  }
  else if ( addrA[0] == 0x28) {
      Serial.print(F("DS18B20 Familie.\n"));
  }
  else {
      Serial.print(F("Geraetefamilie nicht erkannt : 0x"));
      Serial.println(addrA[0],HEX);
      return;
  }
}

// -----------------------------------------------------------------------
// Variablen ins EEPROM schreiben (nur wenn geaendert)
// -----------------------------------------------------------------------
void saveToEEPROM()
{	// alle Werte werden immer auf einmal behandelt, EEPROM.put() regelt, ob
	// der jeweilige Wert tatsächlich geschrieben werden muss (weil geaendert)
	// Damit das funktioniert, muss save und read immer in der gleichen Reihenfolge passieren
	int eeAddress = 0;	// immmer auf Adresse 0 starten
	
	EEPROM.put(eeAddress, iFirstTimeStarted);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i'.
	EEPROM.put(eeAddress, fSollTemp);
	eeAddress += sizeof(float); //Move address to the next byte after float 'f'.
	EEPROM.put(eeAddress, iTempDiff);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i'.
	EEPROM.put(eeAddress, iTempCheckVorlauf);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i'.
	EEPROM.put(eeAddress, iWaitBeforeNextCheck);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i'.
	EEPROM.put(eeAddress, iTempCheckInterval);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i'.
	EEPROM.put(eeAddress, bCooling);
	eeAddress += sizeof(bool); //Move address to the next byte after bool 'b'.
	EEPROM.put(eeAddress, iTempHyst);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i'.
	EEPROM.put(eeAddress, bDCF77);
	eeAddress += sizeof(bool); //Move address to the next byte after bool 'b'.
	EEPROM.put(eeAddress, bRTCtime);
	eeAddress += sizeof(bool); //Move address to the next byte after bool 'b'. 
	EEPROM.put(eeAddress, WoTage);
	eeAddress += sizeof(LuefterZeit); //Move address to the next byte after struct 'LuefterZeit'.
	EEPROM.put(eeAddress, WoEnde);
	eeAddress += sizeof(LuefterZeit); //Move address to the next byte after struct 'LuefterZeit'.
	EEPROM.put(eeAddress, bLuefterMitUhrSteuern);
	eeAddress += sizeof(bool); //Move address to the next byte after bool 'b'. 
}
// -----------------------------------------------------------------------

// -----------------------------------------------------------------------
// Variablen aus EEPROM lesen
// -----------------------------------------------------------------------
void readFromEEPROM()
{	// alle Werte werden immer auf einmal behandelt, per EEPROM.get() 
	// Damit das funktioniert, muss save und read immer in der gleichen Reihenfolge passieren
	int eeAddress = 0;	// immmer auf Adresse 0 starten
	
	EEPROM.get(eeAddress, iFirstTimeStarted);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i' (2 Byte)
	EEPROM.get(eeAddress, fSollTemp);
	eeAddress += sizeof(float); //Move address to the next byte after float 'f' (4 Byte)
	EEPROM.get(eeAddress, iTempDiff);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i' (2 Byte)
	EEPROM.get(eeAddress, iTempCheckVorlauf);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i' (2 Byte)
	EEPROM.get(eeAddress, iWaitBeforeNextCheck);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i' (2 Byte)
	EEPROM.get(eeAddress, iTempCheckInterval);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i' (2 Byte)
	EEPROM.get(eeAddress, bCooling);
	eeAddress += sizeof(bool); //Move address to the next byte after bool 'b' (1 Byte)
	EEPROM.get(eeAddress, iTempHyst);
	eeAddress += sizeof(int); //Move address to the next byte after int 'i' (2 Byte)
	EEPROM.get(eeAddress, bDCF77);
	eeAddress += sizeof(bool); //Move address to the next byte after bool 'b' (1 Byte)
	EEPROM.get(eeAddress, bRTCtime);
	eeAddress += sizeof(bool); //Move address to the next byte after bool 'b' (1 Byte)
	EEPROM.get(eeAddress, WoTage);
	eeAddress += sizeof(LuefterZeit); //Move address to the next byte after struct 'LuefterZeit'.
	EEPROM.get(eeAddress, WoEnde);
	eeAddress += sizeof(LuefterZeit); //Move address to the next byte after struct 'LuefterZeit'.
	EEPROM.get(eeAddress, bLuefterMitUhrSteuern);
	eeAddress += sizeof(bool); //Move address to the next byte after bool 'b' (1 Byte)
}
// -----------------------------------------------------------------------

// -----------------------------------------------------------------------
// Statemachine State in 2. Zeile anzeigen/loeschen
	char cTmpStatenr = 0xB0;	// kleine Zahlen: B0..B9 -> 0..9, C0..C9 -> 0..9 invers
// -----------------------------------------------------------------------
void showStateAnzeige(bool bOn, char cStateNumber)
{
	cTmpStatenr = 0xB0;
	
	if(bStateOnInInit == true) 
	{
		if(bOn == true)
		{
			lcd.setCursor((cStateNumber - 1), 1);	// S1 in Col0, S2 in Col1, ..., S4 in Col3
			// Zeichencode aus Statenummer basteln
			cTmpStatenr += cStateNumber;	// kleine Zahl
			lcd.write(cTmpStatenr);	
			bStateDispOn = true;
			
			if(bLuefterErlaubt == false)
			{
				if(millis() > (uZeitBlinker + 1000))
				{
					lcd.setCursor(0, 1);
					lcd.print(F("Zeit"));
					uZeitBlinker = millis();
				}
			}
		}
		else
		{
			// immer alle States loeschen
			lcd.setCursor(0, 1);
			lcd.print(F("     "));	// 5x Blank (nur 4 States, Pos5 ist Countdown, Pos10 ist Lüfterstate)
			bStateDispOn = false;
		}
	}
}
// -----------------------------------------------------------------------

// -----------------------------------------------------------------------
// CountDown von Timern in 2. Zeile anzeigen
	char cTmpCountDown = 0xB0;	// kleine Zahlen: B0..B9 -> 0..9, C0..C9 -> 0..9 invers
	char cTCDBuff[4] = "";
// -----------------------------------------------------------------------
void showCountdown(int iSecondsLeft)
{
	if(bStateOnInInit == true) 	// Countdown nur im Initscreen anzeigen (nicht im Menue)
	{
		lcd.setCursor(4, 1);			// Countdown Position
		if(iSecondsLeft < 0)
		{
			cTCDBuff[0] = 0x20; 			// Blank
			cTCDBuff[1] = 0x20; 			// Blank
			cTCDBuff[2] = 0x20; 			// Blank
			cTCDBuff[3] = 0x20; 			// Blank
			cTCDBuff[4] = 0x00;				// Abschluss		
		}
		else
		{
			if (iSecondsLeft < 1000){
				cTCDBuff[0] = 0x20; 			// Blank
			}else{
				cTCDBuff[0] = (cTmpCountDown + ((iSecondsLeft/1000)%10));
			}
			if (iSecondsLeft < 100){
				cTCDBuff[1] = 0x20; 			// Blank
			}else{
				cTCDBuff[1] = (cTmpCountDown + ((iSecondsLeft/100)%10));
			}
			if (iSecondsLeft < 10){
				cTCDBuff[2] = 0x20; 			// Blank
			}else{
				cTCDBuff[2] = (cTmpCountDown + ((iSecondsLeft/10)%10));
			}
			cTCDBuff[3] = cTmpCountDown + iSecondsLeft%10;
			cTCDBuff[4] = 0x00;				// Abschluss
		}
		lcd.print(cTCDBuff); 			// print change content
	}
}
// -----------------------------------------------------------------------
		
// -----------------------------------------------------------------------
// Lueftersymbol in 2. Zeile anzeigen
// -----------------------------------------------------------------------
void showLuefter()
{
	// Luefter anzeigen
	if((bShowLuefter == true) && (bStateOnInInit == true))
	{
		lcd.setCursor(9, 1);					// Lueftersymbol
		if(millis() > (uLuefterDispTimer + LUEFTERDISPPHASE))	// alle xxx ms Lueftersymbol wechseln
		{
			switch(iLuefterDispState)
			{
				case 1:	cTmpLuefter = 1;	iLuefterDispState++; break;		// "/"
				case 2:	cTmpLuefter = 2;	iLuefterDispState++; break;		// "-"
				case 3:	cTmpLuefter = 3;	iLuefterDispState++; break;		// "\"
				case 4:	cTmpLuefter = 4;	iLuefterDispState = 1; break;	// "|"
				default: iLuefterDispState = 1; break;
			}
			lcd.write(cTmpLuefter);	
			uLuefterDispTimer = millis();
		}
	}
}
// -----------------------------------------------------------------------
			
// -----------------------------------------------------------------------
// Antennensymbol in 3. Zeile anzeigen
// -----------------------------------------------------------------------
void showAntenne()
{
	// Antenne anzeigen
	if((bDCF77 == true) && (bStateOnInInit == true))
	{
		lcd.setCursor(0, 2); 		// Antennensymbol platzieren
		switch(ucAntenneState)
		{
			case 0: 
				cTmpAntenne = 0x20;		// keine Antenne, Blank
				break;
			case 1:
				cTmpAntenne = 7;			// Antenne ohne Ring
				break;
			case 2:
				if(millis() > (uAntenneDispTimer + ANTENNEDISPPHASE))	// alle xxx ms Antennensymbol wechseln
				{
					switch(ucAntenneDispState)
					{
						case 1:	cTmpAntenne = 5;	ucAntenneDispState = 2; break;	// großer Ring
						case 2:	cTmpAntenne = 6;	ucAntenneDispState = 3; break;	// kleiner Ring
						case 3:	cTmpAntenne = 7;	ucAntenneDispState = 1; break;	// ohne Ring
						default: ucAntenneDispState = 1; break;
					}
					uAntenneDispTimer = millis();
				}
				break;
			case 3: cTmpAntenne = 5;		// Antenne mit statischem Ring
				break;
			default: 
				break;
		}
		lcd.write(cTmpAntenne);	
	}
}
// -----------------------------------------------------------------------
			
// -----------------------------------------------------------------------
// Relais bedienen, ggf. Logging ein/aus
// -----------------------------------------------------------------------
void setRelais(char relaisState)
{
	if(bLuefterErlaubt == true)				// haengt von der Uhrzeit ab
	{
		if(relaisState == ON)
		{
			digitalWrite(RELAIS, HIGH);		// Relais einschalten
		} else
		{
			digitalWrite(RELAIS, LOW);		// Relais ausschalten
		}
	}
	else									// Luefter ist wegen Zeit nicht erlaubt
	{
		digitalWrite(RELAIS, LOW);			// Relais ausschalten
	}
}
// -----------------------------------------------------------------------

// -----------------------------------------------------------------------
// Uhrzeit pruefen, ob Luefter laufen darf
// -----------------------------------------------------------------------
void checkUhrzeit(void)
{
	if(bLuefterMitUhrSteuern == true)
	{
		// Wochentage pruefen, 0..6, 0 = Sonntag
		if((0 == aktUhr.dayOfTheWeek()) || (6 == aktUhr.dayOfTheWeek()))	// Samstag oder Sonntag?
		{
			if((WoEnde.StartMinAbMidnight <= iaktZeitMinAbMidnight) &&
			   (iaktZeitMinAbMidnight <= WoEnde.EndeMinAbMidnight))
			   {
					bLuefterErlaubt = true;	// innerhalb der Grenzen erlaubt
			   }
			   else
			   {
					bLuefterErlaubt = false;				// nicht erlaubt
			   }
		}
		else																// Wochentags
		{
			if((WoTage.StartMinAbMidnight <= iaktZeitMinAbMidnight) &&
			   (iaktZeitMinAbMidnight <= WoTage.EndeMinAbMidnight))
			   {
					bLuefterErlaubt = true;	// innerhalb der Grenzen erlaubt
			   }
			   else
			   {
					bLuefterErlaubt = false;				// nicht erlaubt
			   }
		}
	}
	else
	{
		bLuefterErlaubt = true;				// nicht geprueft, keine Zeitsteuerung, also erlaubt
	}
	
}
// -----------------------------------------------------------------------
// Show Version Dialog
// -----------------------------------------------------------------------
void showVersion(void)
{
#ifdef DEBUGON
	Serial.print(F("showVersion()\n"));
#endif
	// version of TempReg  
  lcd.setCursor(0,0);
  lcd.print(F("TempReg"));
  lcd.setCursor(0,1);
  lcd.print("V");
  lcd.print(TempRegVersionMajor);
  lcd.print(".");
  lcd.print(TempRegVersionMinor);

// version of LOGODisplayAdapter
  lcd.setCursor(0,2);
  lcd.print(F("Adapter"));
  lcd.setCursor(0,3);
  lcd.print(LOGOAdapterPlatine);
}
// -----------------------------------------------------------------------

// -----------------------------------------------------------------------
// Show TimerScreen
// -----------------------------------------------------------------------
void showTimerScreen(void)
{
	// wenn fertig wieder entfernen!!!!
	// lMenuTimeout = millis();		// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
									// der Timer muss bei jedem Tastendruck nachgetriggert werden		
	if(binTimerScreen == false) 
	{
		// lcd.cursor();						// statischer Underline Cursor
		lcd.blink();							// blinkender Cursor
		
		lcd.setCursor(0, 0);
		lcd.print(F("L")); lcd.write(0x81); lcd.print(F("fter an"));	// "Lüfter an"
		lcd.setCursor(1, 1);			// Wochentage, Klein: nicht selektiert, Gross: selektiert
		lcd.print((bmdmdfBlockSel == true ? F("MDMDF") : F("mdmdf")));
		lcd.setCursor(6, 1);
		lcd.print((bssBlockSel == true ? F("SS") : F("ss")));		
		
		lcd.setCursor(0, 2);		
		lcd.print(F("An"));				// Startzeit
		lcd.setCursor(4, 2);
		
		if(bmdmdfBlockSel == true)
		{
			if(WoTage.StartStunde < 10) lcd.print(F("0"));
			lcd.print(WoTage.StartStunde);
		}
		else if(bssBlockSel == true)
		{
			if(WoEnde.StartStunde < 10) lcd.print(F("0"));
			lcd.print(WoEnde.StartStunde);
		}
		else /*if((bmdmdfBlockSel == false) && (bssBlockSel == false))*/
		{
			if(cChangePos == 7)			// "mdmdf"
			{
				if(WoTage.StartStunde < 10) lcd.print(F("0"));
				lcd.print(WoTage.StartStunde);
			}
			else if(cChangePos == 8)	// "ss"
			{
				if(WoEnde.StartStunde < 10) lcd.print(F("0"));
				lcd.print(WoEnde.StartStunde);
			}
		}
	
		lcd.print(F(":"));
		if(bmdmdfBlockSel == true)
		{
			if(WoTage.StartMinute < 10) lcd.print(F("0"));
			lcd.print(WoTage.StartMinute);
		}
		else if(bssBlockSel == true)		
		{			
			if(WoEnde.StartMinute < 10) lcd.print(F("0"));
			lcd.print(WoEnde.StartMinute);
		}
		else if((bmdmdfBlockSel == false) && (bssBlockSel == false))
		{
			if(cChangePos == 7)			// "mdmdf"
			{
				if(WoTage.StartMinute < 10) lcd.print(F("0"));
				lcd.print(WoTage.StartMinute);
			}
			else if(cChangePos == 8)	// "ss"
			{
				if(WoEnde.StartMinute < 10) lcd.print(F("0"));
				lcd.print(WoEnde.StartMinute);
			}
		}

		lcd.setCursor(0, 3);
		lcd.print(F("Aus"));			// Endezeit
		lcd.setCursor(4, 3);
		
		if(bmdmdfBlockSel == true)
		{
			if(WoTage.EndeStunde < 10) lcd.print(F("0"));
			lcd.print(WoTage.EndeStunde);
		}
		else if(bssBlockSel == true)
		{
			if(WoEnde.EndeStunde < 10) lcd.print(F("0"));
			lcd.print(WoEnde.EndeStunde);
		}
		else if((bmdmdfBlockSel == false) && (bssBlockSel == false))
		{
			if(cChangePos == 7)			// "mdmdf"
			{
				if(WoTage.EndeStunde < 10) lcd.print(F("0"));
				lcd.print(WoTage.EndeStunde);
			}
			else if(cChangePos == 8)	// "ss"
			{
				if(WoEnde.EndeStunde < 10) lcd.print(F("0"));
				lcd.print(WoEnde.EndeStunde);
			}
		}

		lcd.print(F(":"));
		if(bmdmdfBlockSel == true)
		{
			if(WoTage.EndeMinute < 10) lcd.print(F("0"));
			lcd.print(WoTage.EndeMinute);
		}
		else if(bssBlockSel == true)
		{
			if(WoEnde.EndeMinute < 10) lcd.print(F("0"));
			lcd.print(WoEnde.EndeMinute);
		}
		else if((bmdmdfBlockSel == false) && (bssBlockSel == false))
		{
			if(cChangePos == 7)			// "mdmdf"
			{
				if(WoTage.EndeMinute < 10) lcd.print(F("0"));
				lcd.print(WoTage.EndeMinute);
			}
			else if(cChangePos == 8)	// "ss"
			{
				if(WoEnde.EndeMinute < 10) lcd.print(F("0"));
				lcd.print(WoEnde.EndeMinute);
			}
		}
		
		binTimerScreen = true;			// der Screen darf nur einmal eufgebaut werden, sonst 
										// wird der Cursor nicht dargestellt
		clastChangePos = cChangePos;
	}
	if(cChangePos != clastChangePos) binTimerScreen = false;
}

